import { Component, ChangeDetectorRef, ElementRef, Input, ChangeDetectionStrategy, ViewChild, ApplicationRef, Injectable, EventEmitter, Output, Directive, HostListener, NgModule } from '@angular/core';
import { TranslateService, TranslateModule } from '@ngx-translate/core';
import { isRequired, FlexyFormJsonMapperService, FlexyFormSchemaService, FlexyFormsModule } from '@ng-flexy/form';
import { FlexyToastsService } from '@ng-flexy/toasts';
import { FlexyLoggerService } from '@ng-flexy/core';
import { moveItemInArray, DragDropModule } from '@angular/cdk/drag-drop';
import { FormControl, Validators, FormBuilder, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { HttpClient } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { template, get, cloneDeep } from 'lodash';
import { BehaviorSubject } from 'rxjs';
import * as momentNs from 'moment';
import { CommonModule } from '@angular/common';
import { FlexyLayoutModule } from '@ng-flexy/layout';
import { TooltipModule, TabsModule, BsDatepickerModule } from 'ngx-bootstrap';
import { NgSelectModule } from '@ng-select/ng-select';
import { TagInputModule } from 'ngx-chips';
import { FlexyJsonImpExpModule } from '@ng-flexy/json-impexp';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormArrayComponent {
    /**
     * @param {?} jsonMapperService
     * @param {?} toasts
     * @param {?} changeDetectorRef
     * @param {?} translate
     * @param {?} logger
     * @param {?} elementRef
     */
    constructor(jsonMapperService, toasts, changeDetectorRef, translate, logger, elementRef) {
        this.jsonMapperService = jsonMapperService;
        this.toasts = toasts;
        this.changeDetectorRef = changeDetectorRef;
        this.translate = translate;
        this.logger = logger;
        this.elementRef = elementRef;
        this.addable = true;
        this.draggable = false;
        this.removable = true;
        this.removeAny = false;
        this.showAddButton = false;
        this.showRemoveButton = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.enableButtons();
        if (this.layoutSchema.formControl) {
            this.isRequired = isRequired(this.layoutSchema.formControl);
        }
        if (this.jsonSchema &&
            this.jsonSchema.validators.minItems &&
            (!this.layoutSchema.items || this.layoutSchema.items.length < this.jsonSchema.validators.minItems)) {
            if (!this.readonly) {
                /** @type {?} */
                const minLength = Math.max(this.layoutSchema.items && this.layoutSchema.items.length ? this.layoutSchema.items.length : 0, this.jsonSchema.validators.minItems);
                for (let i = 0; i < minLength; i++) {
                    this.addNew();
                }
            }
        }
    }
    /**
     * @return {?}
     */
    addNew() {
        if (this.maxItemsExceeded()) {
            this.toasts.error(this.translate.instant('FLEXY_FORM_ARRAY_MAX_VALIDATOR_ERROR', { max: this.jsonSchema.validators.maxItems }));
        }
        else {
            /** @type {?} */
            const newValue = this.jsonSchema.items.children ? {} : null;
            /** @type {?} */
            const control = this.jsonMapperService.createItemControl(this.jsonSchema.items, this.readonly, newValue);
            ((/** @type {?} */ (this.layoutSchema.formControl))).push(control);
            this.layoutSchema.items.push(this.jsonMapperService.createArrayItemSchema(control, this.jsonSchema.items, this.jsonSchema.itemIndexDef, null, this.readonly, {}, newValue, this.layoutSchema.items.length, this.layoutSchema.formControl));
            this.enableButtons();
        }
    }
    /**
     * @param {?} index
     * @return {?}
     */
    removeItem(index) {
        this.toasts.confirm(this.translate.instant('FLEXY_FORM_DELETE_ITEM_CONFIRM'), '', (/**
         * @return {?}
         */
        () => {
            this.layoutSchema.items.splice(index, 1);
            ((/** @type {?} */ (this.layoutSchema.formControl))).removeAt(index);
            this.layoutSchema.formControl.markAsDirty();
        }));
    }
    /**
     * @return {?}
     */
    removeLast() {
        this.toasts.confirm(this.translate.instant('FLEXY_FORM_DELETE_LAST_ITEM_CONFIRM'), '', (/**
         * @return {?}
         */
        () => {
            this.removeLastItem();
        }));
    }
    /**
     * @return {?}
     */
    removeLastItem() {
        if (this.layoutSchema.items.length > 0) {
            /** @type {?} */
            const inx = this.layoutSchema.items.length - 1;
            this.layoutSchema.items.splice(inx, 1);
            ((/** @type {?} */ (this.layoutSchema.formControl))).removeAt(inx);
            ((/** @type {?} */ (this.layoutSchema.formControl))).markAsDirty();
            this.enableButtons();
        }
    }
    /**
     * @return {?}
     */
    maxItemsExceeded() {
        return this.jsonSchema && this.jsonSchema.validators && this.layoutSchema.items.length >= this.jsonSchema.validators.maxItems;
    }
    /**
     * @return {?}
     */
    minItemsExceeded() {
        return (this.jsonSchema &&
            this.jsonSchema.validators &&
            (!this.layoutSchema.items.length || this.layoutSchema.items.length <= this.jsonSchema.validators.minItems));
    }
    /**
     * @return {?}
     */
    focusOnAdded() {
        this.changeDetectorRef.detectChanges();
        /** @type {?} */
        const added = this.elementRef.nativeElement.querySelector('.bottom-button');
        if (added) {
            added.focus();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    drop(event) {
        moveItemInArray(this.layoutSchema.items, event.previousIndex, event.currentIndex);
        this.layoutSchema.formControl.markAsDirty();
    }
    /**
     * @return {?}
     */
    isDraggable() {
        return !this.readonly && this.draggable && this.layoutSchema.items.length > 1;
    }
    /**
     * @private
     * @return {?}
     */
    enableButtons() {
        this.showAddButton = !this.readonly && !this.maxItemsExceeded();
        this.showRemoveButton = !this.readonly && !this.minItemsExceeded();
    }
}
FlexyFormArrayComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-array',
                template: "<fieldset>\n  <h4 *ngIf=\"legend || (!readonly && layoutSchema?.items?.length > 10)\">\n    <span *ngIf=\"legend\">\n      {{ legend }}\n      <span *ngIf=\"isRequired\" class=\"isRequired\" [tooltip]=\"'FLEXY_FORM_FIELD_IS_REQUIRED' | translate\">(<b>*</b>)</span>\n    </span>\n\n    <div class=\"actions\" *ngIf=\"!readonly && layoutSchema?.items?.length > 10\">\n      <button type=\"button\" class=\"btn btn-info btn-outline btn-sm\" [disabled]=\"!showAddButton\" (click)=\"addNew(); focusOnAdded()\">\n        <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n      </button>\n    </div>\n  </h4>\n\n  <ng-container *ngIf=\"layoutSchema && layoutSchema.items\">\n    <div cdkDropList class=\"drag-container schema-items\" [cdkDropListDisabled]=\"!isDraggable()\" (cdkDropListDropped)=\"drop($event)\">\n      <div\n        *ngFor=\"let componentSchema of layoutSchema.items; let index = index\"\n        cdkDrag\n        cdkDragLockAxis=\"y\"\n        class=\"flexy-form-array-draggable-item schema-item\"\n        cdkDragBoundary=\".drag-container\"\n      >\n        <div class=\"schema-item-body\">\n          <flexy-layout [schema]=\"[componentSchema]\" [ngClass]=\"{ 'removable-array-item': !readonly && removable && removeAny }\">\n            <button\n              *ngIf=\"!readonly && removable && removeAny\"\n              type=\"button\"\n              class=\"btn btn-outline delete-array-item e2e-btn-delete-array-item\"\n              tooltip=\"{{ 'FLEXY_FORM_ARRAY_REMOVE_ITEM' | translate }}\"\n              (click)=\"removeItem(index)\"\n            >\n              <i class=\"flexy-icon-remove\"></i>\n            </button>\n          </flexy-layout>\n        </div>\n        <div *ngIf=\"isDraggable()\" class=\"drag-handle\" cdkDragHandle>\n          <button type=\"button\" class=\"btn btn-link\" tooltip=\"{{ 'FLEXY_FORM_ARRAY_REORDER' | translate }}\">\n            <i class=\"flexy-icon-move\"></i>\n          </button>\n        </div>\n      </div>\n    </div>\n\n    <span class=\"actions\" *ngIf=\"!readonly && removable\">\n      <button\n        type=\"button\"\n        class=\"btn btn-info btn-outline btn-sm bottom-button t2e-array-add-btn\"\n        [disabled]=\"!showAddButton\"\n        (click)=\"addNew()\"\n      >\n        <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n      </button>\n      <button\n        *ngIf=\"!removeAny\"\n        type=\"button\"\n        class=\"btn btn-danger btn-outline btn-sm remove-item\"\n        [disabled]=\"!showRemoveButton\"\n        (click)=\"removeLast()\"\n      >\n        {{ 'FLEXY_FORM_ARRAY_REMOVE_LAST' | translate }}\n      </button>\n    </span>\n\n    <flexy-form-field-info *ngIf=\"layoutSchema.formControl?.validator\" [control]=\"layoutSchema.formControl\"></flexy-form-field-info>\n    <p class=\"no-data\" *ngIf=\"!layoutSchema.items.length\">\n      {{ 'FLEXY_FORM_ARRAY_NO_DATA' | translate }}\n    </p>\n  </ng-container>\n</fieldset>\n"
            }] }
];
/** @nocollapse */
FlexyFormArrayComponent.ctorParameters = () => [
    { type: FlexyFormJsonMapperService },
    { type: FlexyToastsService },
    { type: ChangeDetectorRef },
    { type: TranslateService },
    { type: FlexyLoggerService },
    { type: ElementRef }
];
FlexyFormArrayComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    legend: [{ type: Input }],
    jsonSchema: [{ type: Input }],
    readonly: [{ type: Input }],
    addable: [{ type: Input }],
    draggable: [{ type: Input }],
    removable: [{ type: Input }],
    removeAny: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormArrayComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.legend;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.jsonSchema;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.addable;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.draggable;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.removable;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.removeAny;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.isRequired;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.showAddButton;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.showRemoveButton;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.jsonMapperService;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.toasts;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.changeDetectorRef;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.translate;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.logger;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.elementRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormCheckboxComponent {
    /**
     * @return {?}
     */
    toogle() {
        if (!this.readonly) {
            this.layoutSchema.formControl.markAsDirty();
            this.layoutSchema.formControl.setValue(!this.layoutSchema.formControl.value);
        }
    }
}
FlexyFormCheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-checkbox',
                template: `
    <flexy-form-field [control]="layoutSchema.formControl" [label]="label" [description]="description" [ngClass]="{ readonly: readonly }">
      <label (click)="toogle()" role="button">
        <flexy-control-checkbox
          [control]="layoutSchema.formControl"
          [readonly]="readonly"
          [default]="default"
          [onlyLabel]="onlyLabel"
          [faIcon]="faIcon"
        ></flexy-control-checkbox>
        <span *ngIf="suffix"> {{ suffix }}</span>
      </label>
    </flexy-form-field>
  `
            }] }
];
FlexyFormCheckboxComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    label: [{ type: Input }],
    default: [{ type: Input }],
    description: [{ type: Input }],
    readonly: [{ type: Input }],
    onlyLabel: [{ type: Input }],
    faIcon: [{ type: Input }],
    suffix: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.label;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.default;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.description;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.onlyLabel;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.faIcon;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.suffix;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormCheckboxListComponent {
}
FlexyFormCheckboxListComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-checkbox-list',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-checkbox-list
        [name]="layoutSchema.formName"
        [options]="options"
        [readonly]="readonly"
        [default]="default"
        [control]="layoutSchema.formControl"
      >
      </flexy-control-checkbox-list>
    </flexy-form-field>
  `
            }] }
];
FlexyFormCheckboxListComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    options: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.default;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.options;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.label;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.description;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormChipsComponent {
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.default + ')';
            }
        }
    }
}
FlexyFormChipsComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-chips',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [labelIcon]="labelIcon"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <div class="input-group">
        <div class="input-group-addon" *ngIf="prefix">{{ prefix }}</div>
        <flexy-control-chips
          [control]="layoutSchema.formControl"
          [default]="default"
          [readonly]="readonly"
          [autocompleteItems]="autocompleteItems"
          [placeholder]="placeholder"
          [disableDragging]="disableDragging"
          [animations]="animations"
          [onlyFromAutocomplete]="onlyFromAutocomplete"
        ></flexy-control-chips>
        <div class="input-group-addon" *ngIf="suffix">{{ suffix }}</div>
      </div>
    </flexy-form-field>
  `
            }] }
];
FlexyFormChipsComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    autocompleteItems: [{ type: Input }],
    placeholder: [{ type: Input }],
    labelIcon: [{ type: Input }],
    disableDragging: [{ type: Input }],
    animations: [{ type: Input }],
    onlyFromAutocomplete: [{ type: Input }],
    prefix: [{ type: Input }],
    suffix: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormChipsComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.default;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.label;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.description;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.autocompleteItems;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.disableDragging;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.animations;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.onlyFromAutocomplete;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormColorpickerComponent {
}
FlexyFormColorpickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-colorpicker',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-colorpicker [readonly]="readonly" [default]="default" [list]="list" [control]="layoutSchema.formControl">
      </flexy-control-colorpicker>
    </flexy-form-field>
  `
            }] }
];
FlexyFormColorpickerComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    list: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    readonly: [{ type: Input }],
    inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
};
if (false) {
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.default;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.list;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.label;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.description;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormDatepickerComponent {
}
FlexyFormDatepickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-datepicker',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-datepicker
        [control]="layoutSchema.formControl"
        [default]="default"
        [min]="min"
        [max]="max"
        [readonly]="readonly"
        [isDisabled]="isDisabled"
        [placeholder]="placeholder"
        [theme]="theme"
        [format]="format"
        [hideButton]="hideButton"
        [minMode]="minMode"
      >
      </flexy-control-datepicker>
    </flexy-form-field>
  `
            }] }
];
FlexyFormDatepickerComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    placeholder: [{ type: Input }],
    theme: [{ type: Input }],
    readonly: [{ type: Input }],
    isDisabled: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    format: [{ type: Input }],
    hideButton: [{ type: Input }],
    minMode: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.default;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.min;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.max;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.label;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.description;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.format;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.hideButton;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.minMode;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormDaterangepickerComponent {
}
FlexyFormDaterangepickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-daterangepicker',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-daterangepicker
        [control]="layoutSchema.formControl"
        [default]="default"
        [min]="min"
        [max]="max"
        [readonly]="readonly"
        [isDisabled]="isDisabled"
        [placeholder]="placeholder"
        [theme]="theme"
        [format]="format"
        [customClass]="customClass"
      >
      </flexy-control-daterangepicker>
    </flexy-form-field>
  `
            }] }
];
FlexyFormDaterangepickerComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    placeholder: [{ type: Input }],
    theme: [{ type: Input }],
    readonly: [{ type: Input }],
    isDisabled: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    format: [{ type: Input }],
    customClass: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.default;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.min;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.max;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.label;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.description;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.format;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.customClass;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormDraggableSelectComponent {
}
FlexyFormDraggableSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-draggable-select',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [labelIcon]="labelIcon"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-draggable-select
        [control]="layoutSchema.formControl"
        [default]="default"
        [readonly]="readonly"
        [options]="options"
        [placeholder]="placeholder"
        [hideSelected]="hideSelected"
        [enableSearchByValue]="enableSearchByValue"
      >
      </flexy-control-draggable-select>
    </flexy-form-field>
  `
            }] }
];
FlexyFormDraggableSelectComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    options: [{ type: Input }],
    placeholder: [{ type: Input }],
    labelIcon: [{ type: Input }],
    readonly: [{ type: Input }],
    hideSelected: [{ type: Input }],
    enableSearchByValue: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.default;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.label;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.description;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.options;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.hideSelected;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.enableSearchByValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFieldComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.control) {
            this.isRequired = isRequired(this.control);
        }
    }
    /**
     * @return {?}
     */
    focusControl() {
        if (this.control && this.fieldContainerRef && this.fieldContainerRef.nativeElement) {
            /** @type {?} */
            let inputs = this.fieldContainerRef.nativeElement.getElementsByTagName('input');
            if (!inputs[0]) {
                inputs = this.fieldContainerRef.nativeElement.getElementsByTagName('textarea');
            }
            if (inputs[0]) {
                ((/** @type {?} */ (inputs[0]))).focus();
            }
        }
    }
}
FlexyFieldComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-field',
                template: `
    <div
      class="flexy-field"
      [ngClass]="{
        'has-error': control?.dirty && !control.pending && !control.valid,
        'has-success': control?.dirty && !(!control.pending && !control.valid),
        'is-required': isRequired
      }"
      #fieldContainerRef
    >
      <label
        *ngIf="label"
        class="flexy-field-label"
        [ngClass]="{ 'not-empty': control.value || control.value === 0 }"
        (click)="focusControl(); $event.stopPropagation()"
        [attr.data-rel-name]="name"
      >
        <i *ngIf="labelIcon" [class]="labelIcon"></i> {{ label | translate }}{{ label[label.length - 1] !== ':' ? ':' : '' }}
        <span *ngIf="isRequired" class="isRequired" [tooltip]="'FLEXY_FORM_FIELD_IS_REQUIRED' | translate">(<b>*</b>)</span>
      </label>
      <label *ngIf="!label && isRequired" class="isRequired" [tooltip]="'FLEXY_FORM_FIELD_IS_REQUIRED' | translate">(<b>*</b>)</label>

      <div class="flexy-field-control">
        <ng-content></ng-content>
        <flexy-form-field-info *ngIf="control" [control]="control" [description]="description"></flexy-form-field-info>
      </div>
    </div>
  `
            }] }
];
/** @nocollapse */
FlexyFieldComponent.ctorParameters = () => [];
FlexyFieldComponent.propDecorators = {
    control: [{ type: Input }],
    name: [{ type: Input }],
    label: [{ type: Input }],
    labelIcon: [{ type: Input }],
    description: [{ type: Input }],
    fieldContainerRef: [{ type: ViewChild, args: ['fieldContainerRef', { static: false },] }]
};
if (false) {
    /** @type {?} */
    FlexyFieldComponent.prototype.control;
    /** @type {?} */
    FlexyFieldComponent.prototype.name;
    /** @type {?} */
    FlexyFieldComponent.prototype.label;
    /** @type {?} */
    FlexyFieldComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFieldComponent.prototype.description;
    /** @type {?} */
    FlexyFieldComponent.prototype.fieldContainerRef;
    /** @type {?} */
    FlexyFieldComponent.prototype.isRequired;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFieldControlInfoComponent {
    constructor() {
        this.forceDirty = false;
    }
}
FlexyFieldControlInfoComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-field-info',
                template: `
    <div *ngIf="control && (forceDirty || (control.dirty && !control.pending)) && !control.valid" class="field-info-error">
      <ng-container *ngIf="control?.errors">
        <small class="form-text text-danger">
          {{ control.errors | firstError }}
        </small>
      </ng-container>
    </div>
    <div *ngIf="!(control && (forceDirty || (control.dirty && !control.pending)) && !control.valid)" class="field-info-description">
      <small class="form-text text-muted" [innerHtml]="description"></small>
    </div>
  `
            }] }
];
/** @nocollapse */
FlexyFieldControlInfoComponent.ctorParameters = () => [];
FlexyFieldControlInfoComponent.propDecorators = {
    control: [{ type: Input }],
    description: [{ type: Input }],
    forceDirty: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFieldControlInfoComponent.prototype.control;
    /** @type {?} */
    FlexyFieldControlInfoComponent.prototype.description;
    /** @type {?} */
    FlexyFieldControlInfoComponent.prototype.forceDirty;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormFieldsetComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() { }
}
FlexyFormFieldsetComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-fieldset',
                template: "<fieldset>\n  <h4>\n    <span *ngIf=\"legend\">{{ legend }}</span>\n  </h4>\n\n  <flexy-layout [schema]=\"layoutSchema.children\"></flexy-layout>\n  <flexy-form-field-info *ngIf=\"layoutSchema.formControl?.validator\" [control]=\"layoutSchema.formControl\"></flexy-form-field-info>\n</fieldset>\n"
            }] }
];
/** @nocollapse */
FlexyFormFieldsetComponent.ctorParameters = () => [];
FlexyFormFieldsetComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    legend: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormFieldsetComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormFieldsetComponent.prototype.legend;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormGroupComponent {
    /**
     * @param {?} jsonMapperService
     * @param {?} schemaService
     * @param {?} toasts
     * @param {?} translate
     * @param {?} changeDetectorRef
     * @param {?} elementRef
     */
    constructor(jsonMapperService, schemaService, toasts, translate, changeDetectorRef, elementRef) {
        this.jsonMapperService = jsonMapperService;
        this.schemaService = schemaService;
        this.toasts = toasts;
        this.translate = translate;
        this.changeDetectorRef = changeDetectorRef;
        this.elementRef = elementRef;
        this.addable = true;
        this.removable = true;
        this.isRequired = false;
        this.isAddFormVisible = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.layoutSchema && !this.layoutSchema.children) {
            this.layoutSchema.children = [];
        }
        this.isRequired = isRequired(this.layoutSchema.formControl);
        this.addFormControl = new FormControl(this.prepareNewKey(), [
            Validators.required,
            this.keyValidator(this.jsonSchema, this.layoutSchema)
        ]);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    addNew(event) {
        event.stopPropagation();
        event.preventDefault();
        if (this.addFormControl.valid) {
            /** @type {?} */
            const key = this.addFormControl.value;
            this.addFormControl.reset();
            if (this.schemaService.addGroupItemToSchema(this.layoutSchema, key, this.jsonSchema, this.parentGroupName + '.' + key, this.readonly)) {
                /** @type {?} */
                const translateKey = 'key';
                this.toasts.success(this.translate.instant('FLEXY_FORM_GROUP_ADD_ITEM_SUCCESS', { [translateKey]: key }));
                this.addFormControl.setValue(this.prepareNewKey());
                this.isAddFormVisible = false;
            }
        }
    }
    /**
     * @param {?} index
     * @return {?}
     */
    removeItem(index) {
        this.toasts.confirm(this.translate.instant('FLEXY_FORM_DELETE_ITEM_CONFIRM'), '', (/**
         * @return {?}
         */
        () => {
            this.schemaService.removeGroupItemToSchema(this.layoutSchema, index);
        }));
    }
    /**
     * @return {?}
     */
    focusOnNewInput() {
        this.changeDetectorRef.detectChanges();
        /** @type {?} */
        const input = this.elementRef.nativeElement.querySelector('.key-control-text input');
        if (input) {
            input.focus();
        }
    }
    /**
     * @private
     * @return {?}
     */
    prepareNewKey() {
        if (this.jsonSchema.itemKeyGen) {
            return this._randomKey(this.jsonSchema.itemKeyGen);
        }
        else {
            return void 0;
        }
    }
    /**
     * schema is string /(([a-zA-Z0-9]*){([sd])\.?([0-9]*)}+([a-zA-Z0-9]*))/gys
     * supported {s}: string {d}: number with defined size (default :8 max 32) {s.2}
     *
     * for example:
     *
     * AA{d.2}BB{s}
     *
     * generate: AA23BBasewASDq
     * @private
     * @param {?} schema
     * @return {?}
     */
    _randomKey(schema) {
        /** @type {?} */
        let newKey = '';
        /** @type {?} */
        const re = /(([a-zA-Z0-9]*){([sd])\.?([0-9]*)}+([a-zA-Z0-9]*))/gy;
        /** @type {?} */
        let xArray = re.exec(schema);
        while (xArray) {
            newKey += xArray[2] + this._generateRandom(xArray[3], parseInt(xArray[4], 10)) + xArray[5];
            xArray = re.exec(schema);
        }
        return newKey;
    }
    /**
     * @private
     * @param {?} type
     * @param {?} length
     * @return {?}
     */
    _generateRandom(type, length) {
        if (!length) {
            length = 2;
        }
        length = Math.min(length, 32);
        /** @type {?} */
        const shuffleSource = type === 'd' ? '12345678901234567890123456789012345678901234567890' : 'abcdefghijklmnoprqstwxzABCDEFGHIJKLMNOPRQSTWXZ';
        /** @type {?} */
        const s = this._shuffle(shuffleSource.split(''));
        if (length) {
            s.length = length;
        }
        return s.join('');
    }
    /**
     * @private
     * @param {?} array
     * @return {?}
     */
    _shuffle(array) {
        /** @type {?} */
        let counter = array.length;
        // While there are elements in the array
        while (counter > 0) {
            // Pick a random index
            /** @type {?} */
            const index = Math.floor(Math.random() * counter);
            // Decrease counter by 1
            counter--;
            // And swap the last element with it
            /** @type {?} */
            const temp = array[counter];
            array[counter] = array[index];
            array[index] = temp;
        }
        return array;
    }
    // private validateKey(): string {
    //   const err = this.schemaService.validateGroupKey(this.defineKey, this.jsonSchema);
    //   if (err) {
    //     return err;
    //   } else if (Object.keys((<FormGroup>this.layoutSchema.formControl).controls).indexOf(this.defineKey) !== -1) {
    //     return 'exist';
    //   }
    //   return null;
    // }
    /**
     * @private
     * @param {?} jsonSchema
     * @param {?} layoutSchema
     * @return {?}
     */
    keyValidator(jsonSchema, layoutSchema) {
        return (/**
         * @param {?} control
         * @return {?}
         */
        (control) => {
            /** @type {?} */
            const patternReg = jsonSchema.itemKeyPattern ? new RegExp(jsonSchema.itemKeyPattern, 'g') : null;
            if (patternReg && !patternReg.exec(control.value)) {
                return { key_wrong_format: { key: control.value, pattern: patternReg } };
            }
            else if (Object.keys(((/** @type {?} */ (layoutSchema.formControl))).controls).indexOf(control.value) !== -1) {
                return { key_exist: { key: control.value } };
            }
            return null;
        });
    }
}
FlexyFormGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-group',
                template: "<fieldset>\n  <h4>\n    <span *ngIf=\"legend\">\n      {{ legend }}\n      <span *ngIf=\"isRequired\" class=\"isRequired\" [tooltip]=\"'FLEXY_FORM_FIELD_IS_REQUIRED' | translate\">(<b>*</b>)</span>\n    </span>\n    <div class=\"actions\" *ngIf=\"!readonly && addable && layoutSchema?.children?.length > 10\">\n      <button type=\"button\" class=\"btn btn-info btn-outline btn-sm\" (click)=\"isAddFormVisible = true; focusOnNewInput()\">\n        <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n      </button>\n    </div>\n  </h4>\n\n  <ng-container *ngIf=\"layoutSchema && layoutSchema.children\">\n    <ng-container *ngFor=\"let item of layoutSchema.children; let index = index\">\n      <flexy-layout [schema]=\"[item]\">\n        <button\n          *ngIf=\"!readonly && removable\"\n          type=\"button\"\n          class=\"btn btn-outline delete-item e2e-btn-delete-group-item\"\n          (click)=\"removeItem(index)\"\n        >\n          <i class=\"flexy-icon-remove\"></i>\n        </button>\n      </flexy-layout>\n    </ng-container>\n  </ng-container>\n\n  <div class=\"actions add-form-control\" *ngIf=\"!readonly && addable\">\n    <button\n      type=\"button\"\n      *ngIf=\"!isAddFormVisible\"\n      class=\"btn btn-info btn-outline btn-sm\"\n      (click)=\"isAddFormVisible = true; focusOnNewInput()\"\n    >\n      <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n    </button>\n    <flexy-form-field *ngIf=\"isAddFormVisible\" class=\"input-group\" [control]=\"addFormControl\">\n      <div class=\"input-group\">\n        <flexy-control-text class=\"key-control-text\" [control]=\"addFormControl\" (entered)=\"addNew($event)\"></flexy-control-text>\n        <span class=\"input-group-btn input-group-append\">\n          <button\n            type=\"button\"\n            [disabled]=\"!addFormControl.valid\"\n            class=\"btn btn-info btn-outline btn-sm add-item\"\n            (click)=\"addNew($event)\"\n          >\n            {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n          </button>\n        </span>\n        <span class=\"input-group-btn input-group-append\">\n          <button type=\"button\" class=\"btn btn-link btn-sm\" (click)=\"isAddFormVisible = false\">\n            {{ 'FLEXY_FORM_ARRAY_ADD_CANCEL' | translate }}\n          </button>\n        </span>\n      </div>\n    </flexy-form-field>\n  </div>\n\n  <flexy-form-field-info\n    *ngIf=\"layoutSchema && layoutSchema.formControl\"\n    [control]=\"layoutSchema.formControl\"\n    [forceDirty]=\"true\"\n  ></flexy-form-field-info>\n\n  <p class=\"no-data\" *ngIf=\"layoutSchema && !layoutSchema.children.length\">\n    {{ 'FLEXY_FORM_ARRAY_NO_DATA' | translate }}\n  </p>\n</fieldset>\n"
            }] }
];
/** @nocollapse */
FlexyFormGroupComponent.ctorParameters = () => [
    { type: FlexyFormJsonMapperService },
    { type: FlexyFormSchemaService },
    { type: FlexyToastsService },
    { type: TranslateService },
    { type: ChangeDetectorRef },
    { type: ElementRef }
];
FlexyFormGroupComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    legend: [{ type: Input }],
    readonly: [{ type: Input }],
    addable: [{ type: Input }],
    removable: [{ type: Input }],
    jsonSchema: [{ type: Input }],
    parentGroupName: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormGroupComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.legend;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.addable;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.removable;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.jsonSchema;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.parentGroupName;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.isRequired;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.isAddFormVisible;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.addFormControl;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.jsonMapperService;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.schemaService;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.toasts;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.translate;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.changeDetectorRef;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.elementRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormLabelComponent {
}
FlexyFormLabelComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-label',
                template: `
    <label>{{ label }}</label>
  `
            }] }
];
FlexyFormLabelComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    label: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormLabelComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormLabelComponent.prototype.label;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormNumberComponent {
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.refPath && this.refPath.length) {
            /** @type {?} */
            const refControl = this.getRefControl(this.refPath, this.layoutSchema.formControl);
            this.refControl = refControl;
        }
    }
    /**
     * @private
     * @param {?} path
     * @param {?} node
     * @return {?}
     */
    getRefControl(path, node) {
        // tslint:disable-next-line:prefer-for-of
        for (let i = 0; i < path.length; i++) {
            node = node.parent;
        }
        // tslint:disable-next-line:prefer-for-of
        for (let i = 0; i < path.length; i++) {
            node = Number.isInteger(path[i]) ? node.get(Object.keys(node.controls)[path[i]]) : node.controls[path[i]];
        }
        return node;
    }
}
FlexyFormNumberComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-number',
                template: `
    <flexy-form-field
      *ngIf="!refPath || refControl"
      [control]="refControl ? refControl : layoutSchema.formControl"
      [label]="label"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <div class="addon" *ngIf="prefix">{{ prefix }}</div>
      <flexy-control-number
        [control]="refControl ? refControl : layoutSchema.formControl"
        [min]="min"
        [max]="max"
        [step]="step"
        [default]="default"
        [decimal]="decimal"
        [readonly]="readonly"
      >
      </flexy-control-number>
      <div class="addon" *ngIf="suffix">{{ suffix }}</div>
    </flexy-form-field>
  `,
                styles: [`
      .addon {
        display: inline-block;
      }
      flexy-control-number {
        display: inline-block;
      }
    `]
            }] }
];
FlexyFormNumberComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    prefix: [{ type: Input }],
    suffix: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    step: [{ type: Input }],
    readonly: [{ type: Input }],
    decimal: [{ type: Input }],
    refPath: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormNumberComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.default;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.label;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.description;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.min;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.max;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.step;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.decimal;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.refPath;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.refControl;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormPasswordComponent {
}
FlexyFormPasswordComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-password',
                template: `
    <flexy-form-field [control]="layoutSchema.formControl" [label]="label" [description]="description" [ngClass]="{ readonly: readonly }">
      <flexy-control-password
        [minLength]="minLength"
        [maxLength]="maxLength"
        [readonly]="readonly"
        [default]="default"
        [control]="layoutSchema.formControl"
      >
      </flexy-control-password>
    </flexy-form-field>
  `
            }] }
];
FlexyFormPasswordComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    minLength: [{ type: Input }],
    maxLength: [{ type: Input }],
    readonly: [{ type: Input }],
    inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
};
if (false) {
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.default;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.label;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.description;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.minLength;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.maxLength;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormPercentComponent {
}
FlexyFormPercentComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-percent',
                template: `
    <flexy-form-field [control]="layoutSchema.formControl" [label]="label" [description]="description" [ngClass]="{ readonly: readonly }">
      <div class="addon" *ngIf="prefix">{{ prefix }}</div>
      <flexy-control-percent
        [control]="layoutSchema.formControl"
        [min]="min"
        [max]="max"
        [step]="step"
        [default]="default"
        [readonly]="readonly"
      >
      </flexy-control-percent>
      <div class="addon" *ngIf="suffix">{{ suffix }}</div>
    </flexy-form-field>
  `,
                styles: [`
      .addon {
        display: inline-block;
      }
      flexy-control-percent {
        display: inline-block;
      }
    `]
            }] }
];
FlexyFormPercentComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    prefix: [{ type: Input }],
    suffix: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    step: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormPercentComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.default;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.label;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.description;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.min;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.max;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.step;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormRadioListComponent {
}
FlexyFormRadioListComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-radio-list',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-radio-list
        [name]="layoutSchema.formName"
        [options]="options"
        [readonly]="readonly"
        [default]="default"
        [control]="layoutSchema.formControl"
      >
      </flexy-control-radio-list>
    </flexy-form-field>
  `
            }] }
];
FlexyFormRadioListComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    options: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.default;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.options;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.label;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.description;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormSelectComponent {
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.getOptionText(this.default) + ')';
            }
        }
    }
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    getOptionText(value) {
        if (value) {
            /** @type {?} */
            const opt = this.options.find((/**
             * @param {?} item
             * @return {?}
             */
            (item) => '' + item.value === '' + value));
            return opt ? opt.text : '';
        }
        else if (value === null || value === '' || value === void 0) {
            return null;
        }
    }
}
FlexyFormSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-select',
                template: `
    <flexy-form-field [control]="layoutSchema.formControl" [label]="label" [description]="description" [ngClass]="{ readonly: readonly }">
      <div class="input-group">
        <div class="input-group-addon" *ngIf="prefix">{{ prefix }}</div>
        <flexy-control-select
          [control]="layoutSchema.formControl"
          [default]="default"
          [readonly]="readonly"
          [options]="options"
        ></flexy-control-select>
        <div class="input-group-addon" *ngIf="suffix">{{ suffix }}</div>
      </div>
    </flexy-form-field>
  `
            }] }
];
FlexyFormSelectComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    options: [{ type: Input }],
    prefix: [{ type: Input }],
    suffix: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormSelectComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.default;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.label;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.description;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.options;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormSelect2Component {
    /**
     * @param {?} httpClient
     * @param {?} logger
     * @param {?} cdr
     * @param {?} appRef
     */
    constructor(httpClient, logger, cdr, appRef) {
        this.httpClient = httpClient;
        this.logger = logger;
        this.cdr = cdr;
        this.appRef = appRef;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.default + ')';
            }
        }
        if (this.optionsUrl) {
            this.loading = true;
            this.httpClient
                .get(this.optionsUrl)
                .pipe(map((/**
             * @param {?} data
             * @return {?}
             */
            data => {
                /** @type {?} */
                const options = [];
                if (data && Array.isArray(data)) {
                    data.forEach((/**
                     * @param {?} item
                     * @return {?}
                     */
                    item => {
                        options.push({
                            text: '' + this.mapper(item, 'text'),
                            value: this.mapper(item, 'value'),
                            prefixHtml: this.optionsMapper.prefixHtml ? '' + this.mapper(item, 'prefixHtml') : void 0
                        });
                    }));
                }
                return options;
            })))
                .subscribe((/**
             * @param {?} options
             * @return {?}
             */
            options => {
                this.loading = false;
                this.options = options;
                this.cdr.detectChanges();
                // this.appRef.tick();
            }));
        }
    }
    /**
     * @private
     * @param {?} item
     * @param {?} key
     * @return {?}
     */
    mapper(item, key) {
        /** @type {?} */
        const mapper = this.optionsMapper[key];
        if (!mapper) {
            this.logger.warn('Wrong mapper configuration for key', key, this.optionsMapper);
            return;
        }
        // check if template?
        if (mapper.includes('<%')) {
            /** @type {?} */
            const compiled = template(mapper);
            return compiled(item);
        }
        else {
            return get(item, mapper);
        }
    }
}
FlexyFormSelect2Component.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-select2',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [labelIcon]="labelIcon"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <div class="input-group">
        <div class="input-group-addon" *ngIf="prefix">{{ prefix }}</div>
        <flexy-control-select2
          [control]="layoutSchema.formControl"
          [default]="default"
          [readonly]="readonly"
          [options]="options"
          [placeholder]="placeholder"
          [multiple]="multiple"
          [addItem]="addItem"
          [hideSelected]="hideSelected"
          [enableSearchByValue]="enableSearchByValue"
          [loading]="loading"
        ></flexy-control-select2>
        <div class="input-group-addon" *ngIf="suffix">{{ suffix }}</div>
      </div>
    </flexy-form-field>
  `
            }] }
];
/** @nocollapse */
FlexyFormSelect2Component.ctorParameters = () => [
    { type: HttpClient },
    { type: FlexyLoggerService },
    { type: ChangeDetectorRef },
    { type: ApplicationRef }
];
FlexyFormSelect2Component.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    placeholder: [{ type: Input }],
    labelIcon: [{ type: Input }],
    enableSearchByValue: [{ type: Input }],
    options: [{ type: Input }],
    optionsUrl: [{ type: Input }],
    optionsMapper: [{ type: Input }],
    prefix: [{ type: Input }],
    suffix: [{ type: Input }],
    readonly: [{ type: Input }],
    multiple: [{ type: Input }],
    addItem: [{ type: Input }],
    hideSelected: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormSelect2Component.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.default;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.label;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.description;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.placeholder;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.labelIcon;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.enableSearchByValue;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.options;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.optionsUrl;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.optionsMapper;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.prefix;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.suffix;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.readonly;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.multiple;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.addItem;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.hideSelected;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.loading;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.httpClient;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.logger;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.cdr;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.appRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const SCHEMA_COMPONENT_ID_KEY = 'componentId';
class FlexyFormTabsComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.activeTab = this.layoutSchema.children[0] ? this.layoutSchema.children[0][SCHEMA_COMPONENT_ID_KEY] : null;
    }
    /**
     * @param {?} tab
     * @return {?}
     */
    setActiveTab(tab) {
        this.activeTab = tab.id;
    }
}
FlexyFormTabsComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-tabs',
                template: `
    <ng-container *ngIf="layoutSchema && layoutSchema.children">
      <tabset>
        <tab
          *ngFor="let schema of layoutSchema.children"
          [heading]="schema.componentInputs?.legend"
          [id]="schema.componentId"
          [active]="activeTab === schema.componentId"
          (selectTab)="setActiveTab($event)"
        ></tab>
      </tabset>
      <ng-container *ngFor="let schema of layoutSchema.children">
        <flexy-layout *ngIf="activeTab === schema.componentId" [schema]="schema.children"></flexy-layout>
      </ng-container>
    </ng-container>
  `
            }] }
];
/** @nocollapse */
FlexyFormTabsComponent.ctorParameters = () => [];
FlexyFormTabsComponent.propDecorators = {
    layoutSchema: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormTabsComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTabsComponent.prototype.activeTab;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormTagsComponent {
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.default + ')';
            }
        }
    }
}
FlexyFormTagsComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-tags',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-tags
        [control]="layoutSchema.formControl"
        [default]="default"
        [readonly]="readonly"
        [options]="options"
        [placeholder]="placeholder"
      ></flexy-control-tags>
    </flexy-form-field>
  `
            }] }
];
FlexyFormTagsComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    options: [{ type: Input }],
    placeholder: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormTagsComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.default;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.label;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.description;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.options;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormTextComponent {
}
FlexyFormTextComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-text',
                template: `
    <flexy-form-field [control]="layoutSchema.formControl" [label]="label" [description]="description" [ngClass]="{ readonly: readonly }">
      <flexy-control-text
        [minLength]="minLength"
        [maxLength]="maxLength"
        [readonly]="readonly"
        [default]="default"
        [control]="layoutSchema.formControl"
      >
      </flexy-control-text>
    </flexy-form-field>
  `
            }] }
];
FlexyFormTextComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    minLength: [{ type: Input }],
    maxLength: [{ type: Input }],
    readonly: [{ type: Input }],
    inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
};
if (false) {
    /** @type {?} */
    FlexyFormTextComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTextComponent.prototype.default;
    /** @type {?} */
    FlexyFormTextComponent.prototype.label;
    /** @type {?} */
    FlexyFormTextComponent.prototype.description;
    /** @type {?} */
    FlexyFormTextComponent.prototype.minLength;
    /** @type {?} */
    FlexyFormTextComponent.prototype.maxLength;
    /** @type {?} */
    FlexyFormTextComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormTextComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormTextareaComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.inputRef && this.inputRef.nativeElement) {
            this.inputRef.nativeElement.addEventListener('change', (/**
             * @param {?} e
             * @return {?}
             */
            e => {
                this.layoutSchema.formControl.setValue(this.layoutSchema.formControl.value.trim());
            }), false);
        }
    }
}
FlexyFormTextareaComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-textarea',
                template: `
    <flexy-form-field [control]="layoutSchema.formControl" [label]="label" [description]="description" [ngClass]="{ readonly: readonly }">
      <textarea
        #inputRef
        *ngIf="!readonly"
        flexyAutosize
        class="form-control"
        [formControl]="layoutSchema.formControl"
        [attr.placeholder]="default"
      ></textarea>
      <flexy-control-readonly *ngIf="readonly" [value]="layoutSchema.formControl.value" [default]="default"></flexy-control-readonly>
    </flexy-form-field>
  `
            }] }
];
/** @nocollapse */
FlexyFormTextareaComponent.ctorParameters = () => [];
FlexyFormTextareaComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    default: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    readonly: [{ type: Input }],
    inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
};
if (false) {
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.default;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.label;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.description;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyTreeService {
    constructor() {
        this.subject = new BehaviorSubject(null);
    }
    /**
     * @param {?} node
     * @return {?}
     */
    setSelectedId(node) {
        this.subject.next(node);
    }
    /**
     * @return {?}
     */
    onChangeSelected() {
        return this.subject.asObservable();
    }
}
FlexyTreeService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
FlexyTreeService.ctorParameters = () => [];
if (false) {
    /**
     * @type {?}
     * @private
     */
    FlexyTreeService.prototype.subject;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyTreeComponent {
    /**
     * @param {?} treeService
     */
    constructor(treeService) {
        this.treeService = treeService;
        this.tree = [];
        this.selected = new EventEmitter();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.selectedId) {
            this.treeService.setSelectedId(this.findSelectedInChildren(this.tree, changes.selectedId.currentValue));
        }
        if (changes.tree) {
            this.nodes = this.prepareItems(changes.tree.currentValue);
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.selectedSubscription = this.treeService.onChangeSelected().subscribe((/**
         * @param {?} lastSelected
         * @return {?}
         */
        lastSelected => {
            if ((lastSelected && lastSelected.id !== this.selectedId) || (!lastSelected && this.selectedId)) {
                this.selected.emit(lastSelected);
            }
            else if (!lastSelected) {
                this.selected.emit(null);
            }
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.selectedSubscription) {
            this.selectedSubscription.unsubscribe();
        }
    }
    /**
     * @private
     * @param {?} nodes
     * @return {?}
     */
    prepareItems(nodes) {
        if (nodes && nodes.length) {
            return nodes.map((/**
             * @param {?} node
             * @return {?}
             */
            (node) => {
                return {
                    model: node,
                    isExpanded: !!this.findSelectedInChildren(node.children, this.selectedId),
                    isSelected: this.selectedId === node.id,
                    isReadonly: this.readonlyId === node.id,
                    children: node.children ? this.prepareItems(node.children) : null
                };
            }));
        }
        return null;
    }
    /**
     * @private
     * @param {?} tree
     * @param {?} selectedId
     * @return {?}
     */
    findSelectedInChildren(tree, selectedId) {
        /** @type {?} */
        let founded = null;
        if (tree) {
            tree.forEach((/**
             * @param {?} item
             * @return {?}
             */
            item => {
                if (item.id === selectedId) {
                    founded = item;
                    return;
                }
                else if (item.children) {
                    /** @type {?} */
                    const f = this.findSelectedInChildren(item.children, selectedId);
                    if (f) {
                        founded = f;
                        return;
                    }
                }
            }));
        }
        return founded;
    }
}
FlexyTreeComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-tree',
                template: `
    <flexy-tree-node *ngIf="nodes" [nodes]="nodes" [templateRef]="templateRef"></flexy-tree-node>
  `,
                providers: [FlexyTreeService]
            }] }
];
/** @nocollapse */
FlexyTreeComponent.ctorParameters = () => [
    { type: FlexyTreeService }
];
FlexyTreeComponent.propDecorators = {
    tree: [{ type: Input }],
    selectedId: [{ type: Input }],
    readonlyId: [{ type: Input }],
    templateRef: [{ type: Input }],
    selected: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyTreeComponent.prototype.tree;
    /** @type {?} */
    FlexyTreeComponent.prototype.selectedId;
    /** @type {?} */
    FlexyTreeComponent.prototype.readonlyId;
    /** @type {?} */
    FlexyTreeComponent.prototype.templateRef;
    /** @type {?} */
    FlexyTreeComponent.prototype.selected;
    /** @type {?} */
    FlexyTreeComponent.prototype.nodes;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeComponent.prototype.selectedSubscription;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeComponent.prototype.treeService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyTreeNodeComponent {
    /**
     * @param {?} treeService
     */
    constructor(treeService) {
        this.treeService = treeService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._subscription = this.treeService.onChangeSelected().subscribe((/**
         * @param {?} lastSelected
         * @return {?}
         */
        lastSelected => {
            if (this.nodes) {
                this.nodes.forEach((/**
                 * @param {?} item
                 * @return {?}
                 */
                item => {
                    item.isSelected = lastSelected && item.model.id === lastSelected.id;
                }));
            }
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
    }
    /**
     * @param {?} leaf
     * @return {?}
     */
    toggleExpand(leaf) {
        leaf.isExpanded = !leaf.isExpanded;
    }
    /**
     * @param {?} node
     * @return {?}
     */
    toggleSelected(node) {
        node.isSelected = !node.isSelected;
        this.treeService.setSelectedId(node.isSelected ? node.model : null);
    }
}
FlexyTreeNodeComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-tree-node',
                template: `
    <ul class="tree" *ngIf="nodes">
      <li class="node" *ngFor="let item of nodes" [ngClass]="{ active: item.isSelected && !item.isReadonly, readonly: item.isReadonly }">
        <a
          (click)="toggleExpand(item)"
          *ngIf="item.model && item.model.hasChildren()"
          role="button"
          class="fa expand-group t2e-btn-expand"
          [ngClass]="{
            'fa-angle-right': item.model && !item.isExpanded,
            'fa-angle-down': item.model && item.isExpanded
          }"
        ></a>
        <div class="inside t2e-btn-toggle-check" (click)="toggleSelected(item)">
          <span role="button" *ngIf="templateRef">
            <ng-container *ngTemplateOutlet="templateRef; context: { $implicit: item.model, isSelected: item.isSelected }"> </ng-container>
          </span>
          <span *ngIf="!templateRef"> [{{ item.model.id }}] </span>
        </div>
        <div *ngIf="item.isExpanded && item.children">
          <flexy-tree-node [templateRef]="templateRef" [nodes]="item.children"></flexy-tree-node>
        </div>
      </li>
    </ul>
  `
            }] }
];
/** @nocollapse */
FlexyTreeNodeComponent.ctorParameters = () => [
    { type: FlexyTreeService }
];
FlexyTreeNodeComponent.propDecorators = {
    nodes: [{ type: Input }],
    templateRef: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyTreeNodeComponent.prototype.nodes;
    /** @type {?} */
    FlexyTreeNodeComponent.prototype.templateRef;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeNodeComponent.prototype._subscription;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeNodeComponent.prototype.treeService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormTreeSelectComponent {
    /**
     * @return {?}
     */
    ngOnInit() { }
}
FlexyFormTreeSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-tree-select',
                template: `
    <flexy-form-field
      [control]="layoutSchema.formControl"
      [name]="layoutSchema.formName"
      [label]="label"
      [labelIcon]="labelIcon"
      [description]="description"
      [ngClass]="{ readonly: readonly }"
    >
      <flexy-control-tree-select
        [control]="layoutSchema.formControl"
        [dataTree]="dataTree"
        [default]="default"
        [readonlyId]="readonlyId"
        [labelKey]="labelKey"
        [readonly]="readonly"
      >
      </flexy-control-tree-select>
    </flexy-form-field>
  `
            }] }
];
FlexyFormTreeSelectComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    dataTree: [{ type: Input }],
    default: [{ type: Input }],
    readonlyId: [{ type: Input }],
    labelKey: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    labelIcon: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.dataTree;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.default;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.readonlyId;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.labelKey;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.label;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.description;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlCheckboxComponent {
    constructor() {
        this.isValueDefined = false;
        this.isDefaultDefined = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.isValueDefined = this.control && this.control.value !== null && this.control.value !== void 0;
        this.isDefaultDefined = this.default !== void 0;
        if (!this.readonly && !this.isValueDefined && this.isDefaultDefined) {
            this.control.setValue(this.default);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFocus(event) {
        event.target.onkeydown = (/**
         * @param {?} keyEvent
         * @return {?}
         */
        keyEvent => {
            if (keyEvent.code === 'Space') {
                keyEvent.preventDefault();
                keyEvent.stopPropagation();
                this.toogle();
            }
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onBlur(event) {
        event.target.onkeydown = null;
    }
    /**
     * @return {?}
     */
    toogle() {
        this.control.markAsDirty();
        this.control.setValue(!this.control.value);
    }
}
FlexyControlCheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-control-checkbox',
                template: `
    <a
      *ngIf="!readonly && onlyLabel"
      (focus)="onFocus($event)"
      (blur)="onBlur($event)"
      (click)="toogle()"
      [ngClass]="{ active: control.value }"
    >
      <i *ngIf="faIcon" [class]="faIcon"></i> {{ onlyLabel }}
    </a>
    <a
      role="checkbox"
      tabindex="0"
      *ngIf="!readonly && !onlyLabel"
      class="fa"
      (focus)="onFocus($event)"
      (blur)="onBlur($event)"
      (click)="toogle(); $event.stopPropagation()"
      [ngClass]="{
        'fa-check-square-o': control.value,
        'fa-square-o': !control.value,
        'ng-dirty': control.dirty,
        'ng-invalid': !control.valid
      }"
      role="button"
    ></a>
    <span
      *ngIf="readonly && isValueDefined"
      class="fa"
      [ngClass]="{
        'fa-check': control.value === true,
        'fa-minus': control.value === false
      }"
    ></span>
    <div *ngIf="isDefaultDefined && !isValueDefined">
      <small>
        <span *ngIf="readonly" class="fa" [ngClass]="{ 'fa-check': default, 'fa-minus': !default }"></span>
        <span *ngIf="!readonly && !control.dirty">default</span>
      </small>
    </div>
  `
            }] }
];
/** @nocollapse */
FlexyControlCheckboxComponent.ctorParameters = () => [];
FlexyControlCheckboxComponent.propDecorators = {
    control: [{ type: Input }],
    readonly: [{ type: Input }],
    default: [{ type: Input }],
    onlyLabel: [{ type: Input }],
    faIcon: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.control;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.default;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.onlyLabel;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.faIcon;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.isValueDefined;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.isDefaultDefined;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlCheckboxListComponent {
    /**
     * @param {?} formBuilder
     */
    constructor(formBuilder) {
        this.formBuilder = formBuilder;
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.initCheckboxesGroup();
        this.setDefaultsLabels();
        this.setValueLabels();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.controlChangesSubscription) {
            this.controlChangesSubscription.unsubscribe();
        }
        if (this.localValuesChangesSubscription) {
            this.localValuesChangesSubscription.unsubscribe();
        }
    }
    /**
     * @private
     * @return {?}
     */
    initCheckboxesGroup() {
        /** @type {?} */
        const formConfig = {};
        if (this.options) {
            /** @type {?} */
            const values = this.control.value || [];
            this.options.forEach((/**
             * @param {?} option
             * @return {?}
             */
            option => {
                formConfig[option.value] = [values.includes(option.value), []];
            }));
        }
        this.localGroupControl = this.formBuilder.group(formConfig);
        this.controlChangesSubscription = this.control.valueChanges.subscribe((/**
         * @param {?} values
         * @return {?}
         */
        values => {
            if (this.localValuesChangesSubscription) {
                this.localValuesChangesSubscription.unsubscribe();
            }
            this.options.forEach((/**
             * @param {?} option
             * @return {?}
             */
            option => {
                this.localGroupControl.get(option.value).setValue(values && values.includes(option.value) ? true : null);
            }));
            this.subscribeLocalChanges();
        }));
        this.subscribeLocalChanges();
    }
    /**
     * @private
     * @return {?}
     */
    subscribeLocalChanges() {
        this.localValuesChangesSubscription = this.localGroupControl.valueChanges.subscribe((/**
         * @param {?} values
         * @return {?}
         */
        values => {
            this.updateControl();
        }));
    }
    /**
     * @private
     * @return {?}
     */
    setDefaultsLabels() {
        if (this.default) {
            this.defaultsLabels = this.options
                .filter((/**
             * @param {?} option
             * @return {?}
             */
            option => this.default.includes(option.value)))
                .map((/**
             * @param {?} option
             * @return {?}
             */
            option => option.text))
                .join(', ');
        }
    }
    /**
     * @private
     * @return {?}
     */
    setValueLabels() {
        if (this.options) {
            this.valuesLabels = this.options
                .filter((/**
             * @param {?} option
             * @return {?}
             */
            option => this.control.value && this.control.value.includes(option.value)))
                .map((/**
             * @param {?} option
             * @return {?}
             */
            option => option.text))
                .join(', ');
        }
    }
    /**
     * @private
     * @return {?}
     */
    updateControl() {
        /** @type {?} */
        const val = this.options.filter((/**
         * @param {?} option
         * @return {?}
         */
        option => !!this.localGroupControl.get(option.value).value)).map((/**
         * @param {?} option
         * @return {?}
         */
        option => option.value));
        this.control.setValue(val.length ? val : null);
        this.control.markAsDirty();
        this.setValueLabels();
        this.changed.emit(this.control.value);
    }
}
FlexyControlCheckboxListComponent.decorators = [
    { type: Component, args: [{
                template: `
    <div *ngIf="!readonly" class="input-group form-control">
      <div *ngFor="let option of options; let i = index" class="checkboxItem">
        <input
          type="checkbox"
          [id]="name + '-' + option.value"
          [value]="option.value"
          [formControl]="localGroupControl.get(option.value)"
        /><label for="{{ name + '-' + option.value }}">{{ option.text }}</label>
      </div>
    </div>
    <flexy-control-readonly *ngIf="readonly" [value]="valuesLabels" [default]="defaultsLabels"></flexy-control-readonly>
  `,
                selector: 'flexy-control-checkbox-list'
            }] }
];
/** @nocollapse */
FlexyControlCheckboxListComponent.ctorParameters = () => [
    { type: FormBuilder }
];
FlexyControlCheckboxListComponent.propDecorators = {
    control: [{ type: Input }],
    name: [{ type: Input }],
    default: [{ type: Input }],
    options: [{ type: Input }],
    readonly: [{ type: Input }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.control;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.name;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.default;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.options;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.changed;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.localGroupControl;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.valuesLabels;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.defaultsLabels;
    /**
     * @type {?}
     * @private
     */
    FlexyControlCheckboxListComponent.prototype.controlChangesSubscription;
    /**
     * @type {?}
     * @private
     */
    FlexyControlCheckboxListComponent.prototype.localValuesChangesSubscription;
    /**
     * @type {?}
     * @private
     */
    FlexyControlCheckboxListComponent.prototype.formBuilder;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlChipsComponent {
    constructor() {
        this.autocompleteItems = [];
        this.disableDragging = false;
        this.animations = false;
        this.onlyFromAutocomplete = false;
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.animationDuration = !this.animations ? { enter: '0ms', leave: '0ms' } : { enter: '250ms', leave: '150ms' };
        this.setFormControl();
    }
    /**
     * @return {?}
     */
    onChange() {
        this.control.setValue(this.chipsFormControl.value.map((/**
         * @param {?} val
         * @return {?}
         */
        val => (val === 'null' ? null : val))));
        this.control.markAsDirty();
    }
    /**
     * @private
     * @return {?}
     */
    setFormControl() {
        /** @type {?} */
        const control = cloneDeep(this.control);
        control.setValue(control.value ? control.value.map((/**
         * @param {?} val
         * @return {?}
         */
        val => String(val))) : []);
        this.chipsFormControl = control;
        this.autocompleteItems = this.autocompleteItems && this.autocompleteItems.length ? this.autocompleteItems : control.value;
    }
}
FlexyControlChipsComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-control-chips',
                template: `
    <tag-input
      *ngIf="!readonly"
      [formControl]="chipsFormControl"
      [editable]="true"
      [dragZone]="!disableDragging ? 'dragzone' : null"
      [modelAsStrings]="true"
      [animationDuration]="animationDuration"
      [placeholder]="placeholder"
      [secondaryPlaceholder]="placeholder"
      [theme]="'ngx-chips-custom-theme'"
      [addOnBlur]="true"
      [onlyFromAutocomplete]="onlyFromAutocomplete"
      (onTagEdited)="onChange()"
      (onRemove)="onChange()"
      (onAdd)="onChange()"
    >
      <tag-input-dropdown [autocompleteItems]="autocompleteItems" [showDropdownIfEmpty]="true" [appendToBody]="false"></tag-input-dropdown>
    </tag-input>
    <flexy-control-readonly *ngIf="readonly" [value]="chipsFormControl?.value.join(', ')" [default]="default"> </flexy-control-readonly>
  `
            }] }
];
FlexyControlChipsComponent.propDecorators = {
    control: [{ type: Input }],
    autocompleteItems: [{ type: Input }],
    default: [{ type: Input }],
    readonly: [{ type: Input }],
    placeholder: [{ type: Input }],
    disableDragging: [{ type: Input }],
    animations: [{ type: Input }],
    onlyFromAutocomplete: [{ type: Input }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlChipsComponent.prototype.control;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.autocompleteItems;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.default;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.disableDragging;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.animations;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.onlyFromAutocomplete;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.changed;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.animationDuration;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.chipsFormControl;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlColorpickerComponent {
    constructor() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
}
FlexyControlColorpickerComponent.decorators = [
    { type: Component, args: [{
                template: `
    <div class="form-control color-picker-container">
      <div class="form-control color-picker-input">
        <input
          *ngIf="!readonly"
          class="notreadonly"
          type="color"
          list="presetColors"
          [value]="control.value ? control.value : default"
          [formControl]="control"
          (focus)="focused.emit($event)"
          (click)="clicked.emit($event)"
          (changed)="changed.emit($event)"
        />
        <datalist id="presetColors" *ngIf="!readonly">
          <option *ngFor="let color of list">{{ color }}</option>
        </datalist>
        <input type="color" *ngIf="readonly" [value]="control.value ? control.value : default" disabled="true" />
      </div>
    </div>
    <flexy-control-readonly *ngIf="readonly" [value]="control.value" [default]="default"></flexy-control-readonly>
  `,
                selector: 'flexy-control-colorpicker'
            }] }
];
FlexyControlColorpickerComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    list: [{ type: Input }],
    readonly: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.control;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.default;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.list;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.focused;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DATEPICKER_THEMES = ['theme-default', 'theme-green', 'theme-blue', 'theme-dark-blue', 'theme-red', 'theme-orange'];
/** @type {?} */
const BS_DATEPICKER_MODES = {
    month: 'month',
    quarter: 'month',
    year: 'year',
    day: 'day'
};
class FlexyControlDatepickerComponent {
    constructor() {
        this.focus = new EventEmitter();
        this.click = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.configDatepicker();
    }
    /**
     * @private
     * @return {?}
     */
    configDatepicker() {
        this.minDate = this.min ? new Date(this.min) : null;
        this.maxDate = this.max ? new Date(this.max) : null;
        this.bsConfig = {
            minMode: this.minMode && BS_DATEPICKER_MODES[this.minMode] ? BS_DATEPICKER_MODES[this.minMode] : 'day',
            dateInputFormat: this.format ? this.format.toUpperCase() : 'YYYY-MM-DD',
            containerClass: [DATEPICKER_THEMES.includes(this.theme) ? this.theme : 'theme-default', 'mode-' + this.minMode]
        };
    }
}
FlexyControlDatepickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-control-datepicker',
                template: `
    <div class="input-group" *ngIf="!readonly">
      <input
        #dp="bsDatepicker"
        class="form-control"
        type="text"
        [isDisabled]="isDisabled"
        [formControl]="control"
        [attr.placeholder]="placeholder"
        [bsConfig]="bsConfig"
        [minDate]="minDate"
        [maxDate]="maxDate"
        (focus)="focus.emit($event)"
        (click)="click.emit($event)"
        (bsValueChange)="changed.emit($event)"
        bsDatepicker
      />
      <span class="input-group-btn">
        <button *ngIf="!hideButton" class="btn btn-primary" (click)="dp.toggle(); $event.stopPropagation()">
          <span class="flexy-icon-calendar"></span>
        </button>
      </span>
    </div>
    <flexy-control-readonly *ngIf="readonly" [value]="control.value | date: format" [default]="default"> </flexy-control-readonly>
  `
            }] }
];
FlexyControlDatepickerComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    placeholder: [{ type: Input }],
    theme: [{ type: Input }],
    format: [{ type: Input }],
    readonly: [{ type: Input }],
    isDisabled: [{ type: Input }],
    hideButton: [{ type: Input }],
    minMode: [{ type: Input }],
    focus: [{ type: Output }],
    click: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.bsConfig;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.bsValue;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.minDate;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.maxDate;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.defaultFormatted;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.control;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.default;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.min;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.max;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.format;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.hideButton;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.minMode;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.focus;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.click;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const moment = momentNs;
/** @type {?} */
const DATEPICKER_THEMES$1 = ['theme-default', 'theme-green', 'theme-blue', 'theme-dark-blue', 'theme-red', 'theme-orange'];
class FlexyControlDaterangepickerComponent {
    constructor() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.configDatepicker();
        this.setReadonlyValue();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onChange(event) {
        this.setReadonlyValue();
        this.changed.emit(event);
    }
    /**
     * @private
     * @return {?}
     */
    configDatepicker() {
        if (this.default && this.default.start && this.default.end) {
            this.defaultRange = this.default.start + ' - ' + this.default.end;
        }
        this.minDate = this.min ? new Date(this.min) : null;
        this.maxDate = this.max ? new Date(this.max) : null;
        this.bsConfig = {
            rangeInputFormat: this.format ? this.format.toUpperCase() : 'YYYY-MM-DD',
            containerClass: [DATEPICKER_THEMES$1.includes(this.theme) ? this.theme : 'theme-default', this.customClass]
        };
    }
    /**
     * @private
     * @return {?}
     */
    setReadonlyValue() {
        if (this.control.value && this.format) {
            this.readonlyValue = this.control.value.map((/**
             * @param {?} val
             * @return {?}
             */
            val => moment(val).format(this.format.toUpperCase()))).join(' - ');
        }
    }
}
FlexyControlDaterangepickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-control-daterangepicker',
                template: `
    <div class="input-group" *ngIf="!readonly">
      <input
        #drp="bsDaterangepicker"
        class="form-control"
        type="text"
        [isDisabled]="isDisabled"
        [formControl]="control"
        [attr.placeholder]="placeholder"
        [bsConfig]="bsConfig"
        [minDate]="minDate"
        [maxDate]="maxDate"
        (focus)="focused.emit($event)"
        (click)="clicked.emit($event)"
        (bsValueChange)="onChange($event)"
        bsDaterangepicker
      />
      <span class="input-group-btn">
        <button class="btn btn-primary" (click)="drp.toggle(); $event.stopPropagation()">
          <span class="flexy-icon-calendar"></span>
        </button>
      </span>
    </div>
    <flexy-control-readonly *ngIf="readonly" [value]="readonlyValue" [default]="defaultRange"></flexy-control-readonly>
  `
            }] }
];
FlexyControlDaterangepickerComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    placeholder: [{ type: Input }],
    theme: [{ type: Input }],
    format: [{ type: Input }],
    readonly: [{ type: Input }],
    isDisabled: [{ type: Input }],
    customClass: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.control;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.default;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.min;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.max;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.format;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.customClass;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.focused;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.changed;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.bsConfig;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.bsRangeValue;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.minDate;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.maxDate;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.defaultRange;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.readonlyValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlDraggableSelectComponent {
    constructor() {
        this.optionsMap = {};
        this.showSelect = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.options.forEach((/**
         * @param {?} option
         * @return {?}
         */
        option => {
            this.optionsMap[option.value] = option.text;
        }));
    }
    /**
     * @param {?} event
     * @return {?}
     */
    drop(event) {
        moveItemInArray(this.control.value, event.previousIndex, event.currentIndex);
        this.control.markAsDirty();
    }
    /**
     * @param {?} index
     * @return {?}
     */
    removeItem(index) {
        /** @type {?} */
        const value = this.control.value;
        value.splice(index, 1);
        this.control.setValue(value);
        this.control.markAsDirty();
    }
    /**
     * @return {?}
     */
    isDraggable() {
        return !this.readonly && this.control.value && this.control.value.length > 1;
    }
}
FlexyControlDraggableSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-control-draggable-select',
                template: `
    <div class="draggable-list-container">
      <table cdkDropList class="drag-container" [cdkDropListDisabled]="!isDraggable()" (cdkDropListDropped)="drop($event)">
        <tr
          *ngFor="let value of control.value; let index = index"
          cdkDrag
          cdkDragLockAxis="y"
          class="flexy-form-select-draggable-item"
          [ngClass]="{ draggable: isDraggable() }"
          cdkDragBoundary=".drag-container"
        >
          <td>
            {{ optionsMap[value] }}
            <span *ngIf="!optionsMap[value]" class="unknown-value">{{ 'FLEXY_FORM_UNKNOWN_VALUE' | translate: { value: value } }}</span>
          </td>
          <td class="draggable-item-remove">
            <button
              *ngIf="!readonly"
              type="button"
              class="btn btn-outline delete-select-item e2e-btn-delete-select-item"
              tooltip="{{ 'FLEXY_FORM_ARRAY_REMOVE_ITEM' | translate }}"
              (click)="removeItem(index)"
            >
              <i class="flexy-icon-remove"></i>
            </button>
          </td>
        </tr>
      </table>
    </div>
    <button
      *ngIf="!readonly && !showSelect"
      type="button"
      class="btn btn-info btn-outline btn-sm bottom-button"
      (click)="showSelect = !showSelect"
    >
      <i class="flexy-icon-plus"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}
    </button>
    <flexy-control-select2
      *ngIf="showSelect"
      [control]="control"
      [default]="default"
      [options]="options"
      [multiple]="true"
      [hideSelected]="hideSelected"
      [enableSearchByValue]="enableSearchByValue"
      (changed)="showSelect = false"
    ></flexy-control-select2>
  `
            }] }
];
FlexyControlDraggableSelectComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    options: [{ type: Input }],
    placeholder: [{ type: Input }],
    hideSelected: [{ type: Input }],
    enableSearchByValue: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.control;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.default;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.options;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.hideSelected;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.enableSearchByValue;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.optionsMap;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.showSelect;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlNumberComponent {
    constructor() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
}
FlexyControlNumberComponent.decorators = [
    { type: Component, args: [{
                template: `
    <input
      *ngIf="!readonly"
      class="form-control"
      type="number"
      [formControl]="control"
      [attr.min]="min"
      [attr.max]="max"
      [attr.step]="step ? step : 1"
      [attr.placeholder]="default"
      (focus)="focused.emit($event)"
      (click)="clicked.emit($event)"
      (change)="changed.emit($event)"
    />
    <flexy-control-readonly
      *ngIf="readonly"
      [value]="decimal ? (control.value | number: decimal) : control.value"
      [default]="default"
    ></flexy-control-readonly>
  `,
                selector: 'flexy-control-number',
                styles: [`
      input {
        width: 10em;
      }
    `]
            }] }
];
FlexyControlNumberComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    step: [{ type: Input }],
    readonly: [{ type: Input }],
    decimal: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlNumberComponent.prototype.control;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.default;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.min;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.max;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.step;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.decimal;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.focused;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlPasswordComponent {
    constructor() {
        this.autocomplete = '';
        this.onFocus = new EventEmitter();
        this.onClick = new EventEmitter();
        this.onChanged = new EventEmitter();
    }
}
FlexyControlPasswordComponent.decorators = [
    { type: Component, args: [{
                template: `
    <input
      *ngIf="!readonly"
      class="form-control"
      type="password"
      [formControl]="control"
      [attr.minLength]="minLength"
      [attr.maxLength]="maxLength"
      [attr.placeholder]="default"
      [attr.autocomplete]="'new-password'"
      (focus)="onFocus.emit($event)"
      (click)="onClick.emit($event)"
      (change)="onChanged.emit($event)"
    />
    <flexy-control-readonly *ngIf="readonly" [value]="control.value" [default]="default"></flexy-control-readonly>
  `,
                selector: 'flexy-control-password'
            }] }
];
FlexyControlPasswordComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    minLength: [{ type: Input }],
    maxLength: [{ type: Input }],
    readonly: [{ type: Input }],
    autocomplete: [{ type: Input }],
    onFocus: [{ type: Output }],
    onClick: [{ type: Output }],
    onChanged: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.control;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.default;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.minLength;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.maxLength;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.autocomplete;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.onFocus;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.onClick;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.onChanged;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlPercentComponent {
    constructor() {
        this.onFocus = new EventEmitter();
        this.onClick = new EventEmitter();
        this.onChanged = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.percentValue = typeof this.control.value === 'number' ? this.control.value * 100 : null;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onChange(event) {
        this.control.setValue(this.percentValue / 100);
        this.onChanged.emit(event);
    }
}
FlexyControlPercentComponent.decorators = [
    { type: Component, args: [{
                template: `
    <input
      *ngIf="!readonly"
      class="form-control"
      type="number"
      [(ngModel)]="percentValue"
      [attr.min]="min"
      [attr.max]="max"
      [attr.step]="step ? step : 1"
      [attr.placeholder]="default"
      (focus)="onFocus.emit($event)"
      (click)="onClick.emit($event)"
      (change)="onChange($event)"
    />
    <flexy-control-readonly *ngIf="readonly" [value]="percentValue" [default]="default"></flexy-control-readonly>
  `,
                selector: 'flexy-control-percent',
                styles: [`
      input {
        width: 10em;
      }
    `]
            }] }
];
FlexyControlPercentComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    step: [{ type: Input }],
    readonly: [{ type: Input }],
    onFocus: [{ type: Output }],
    onClick: [{ type: Output }],
    onChanged: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlPercentComponent.prototype.control;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.default;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.min;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.max;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.step;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.onFocus;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.onClick;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.onChanged;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.percentValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlRadioListComponent {
    constructor() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
}
FlexyControlRadioListComponent.decorators = [
    { type: Component, args: [{
                template: `
    <div *ngIf="!readonly" class="input-group form-control">
      <div *ngFor="let option of options" class="radioItem">
        <input
          type="radio"
          [formControl]="control"
          [id]="name + '-' + option.value"
          [name]="name"
          [value]="option.value"
          (focus)="focused.emit($event)"
          (click)="clicked.emit($event)"
          (change)="changed.emit($event)"
        /><label for="{{ name + '-' + option.value }}">{{ option.text }}</label>
      </div>
    </div>
    <flexy-control-readonly *ngIf="readonly" [value]="control.value" [default]="default"></flexy-control-readonly>
  `,
                selector: 'flexy-control-radio-list'
            }] }
];
FlexyControlRadioListComponent.propDecorators = {
    control: [{ type: Input }],
    name: [{ type: Input }],
    default: [{ type: Input }],
    options: [{ type: Input }],
    readonly: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.control;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.name;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.default;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.options;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.focused;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlReadonlyComponent {
    constructor() {
        this.isHtml = false;
    }
}
FlexyControlReadonlyComponent.decorators = [
    { type: Component, args: [{
                template: `
    <span>
      <ng-container *ngIf="!isHtml">{{ value }}</ng-container>
      <span *ngIf="isHtml" [innerHTML]="value"></span>
      <i *ngIf="value === null || value === undefined">{{ default }}</i>
    </span>
  `,
                selector: 'flexy-control-readonly'
            }] }
];
FlexyControlReadonlyComponent.propDecorators = {
    value: [{ type: Input }],
    default: [{ type: Input }],
    isHtml: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyControlReadonlyComponent.prototype.value;
    /** @type {?} */
    FlexyControlReadonlyComponent.prototype.default;
    /** @type {?} */
    FlexyControlReadonlyComponent.prototype.isHtml;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlSelectComponent {
    constructor() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /** @type {?} */
        const optionsData = [{ value: null, text: '--' }];
        if (this.options) {
            optionsData.push(...this.options.map((/**
             * @param {?} item
             * @return {?}
             */
            item => {
                /** @type {?} */
                let itemData;
                if (typeof item === 'object' && item.hasOwnProperty('value')) {
                    itemData = item;
                }
                else {
                    itemData = {
                        value: item,
                        text: '' + item
                    };
                }
                return itemData;
            })));
        }
        this.optionsData = optionsData;
    }
    /**
     * @param {?} data
     * @return {?}
     */
    onChanged(data) {
        this.changed.emit(data.value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    getOptionText(value) {
        if (value || value === 0) {
            /** @type {?} */
            const opt = this.options.find((/**
             * @param {?} item
             * @return {?}
             */
            (item) => '' + item.value === '' + value));
            return opt ? opt.text : '';
        }
        else if (value === null || value === '' || value === void 0) {
            return null;
        }
    }
}
FlexyControlSelectComponent.decorators = [
    { type: Component, args: [{
                template: `
    <select
      *ngIf="!readonly"
      class="form-control"
      [formControl]="control"
      [attr.placeholder]="getOptionText(default)"
      (focus)="focused.emit($event)"
      (click)="clicked.emit($event)"
      (change)="onChanged($event)"
    >
      <option *ngFor="let option of optionsData" [ngValue]="option.value">
        {{ option.text }}
      </option>
    </select>
    <flexy-control-readonly *ngIf="readonly" [value]="getOptionText(control.value)" [default]="getOptionText(default)">
    </flexy-control-readonly>
  `,
                selector: 'flexy-control-select'
            }] }
];
FlexyControlSelectComponent.propDecorators = {
    control: [{ type: Input }],
    options: [{ type: Input }],
    default: [{ type: Input }],
    readonly: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlSelectComponent.prototype.control;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.options;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.default;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.focused;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.changed;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.optionsData;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlSelect2Component {
    constructor() {
        this.loading = false; // no	You can set the loading state from the outside (e.g. async items loading)
        // no	You can set the loading state from the outside (e.g. async items loading)
        this.loadingText = 'Loading...'; // Loading...	no	Set custom text when for loading items
        // Loading...	no	Set custom text when for loading items
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
        this.virtualScroll = false; // Enable virtual scroll for better performance when rendering a lot of data
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.options) {
            /** @type {?} */
            const optionsData = [];
            if (this.options) {
                optionsData.push(...this.options.map((/**
                 * @param {?} item
                 * @return {?}
                 */
                item => {
                    /** @type {?} */
                    let itemData;
                    if (typeof item === 'object' && item.hasOwnProperty('value')) {
                        itemData = item;
                    }
                    else {
                        itemData = {
                            value: item,
                            text: '' + item
                        };
                    }
                    return itemData;
                })));
            }
            this.virtualScroll = optionsData.length > 100;
            this.optionsData = optionsData;
        }
    }
    /**
     * @param {?} data
     * @return {?}
     */
    onChange(data) {
        /** @type {?} */
        const value = data ? (Array.isArray(data) ? data.map((/**
         * @param {?} el
         * @return {?}
         */
        el => el.value)) : data.value) : null;
        this.control.setValue(value);
        this.changed.emit(value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    getReadonlyInfo(value) {
        if (value && this.addItem) {
            return value;
        }
        if ((value || value === 0) && this.optionsData) {
            /** @type {?} */
            const opt = this.optionsData.find((/**
             * @param {?} item
             * @return {?}
             */
            (item) => '' + item.value === '' + value));
            return opt ? opt.text : '';
        }
        else if (value === null || value === '' || value === void 0) {
            return null;
        }
    }
    /**
     * @param {?} term
     * @param {?} item
     * @return {?}
     */
    customSearchFn(term, item) {
        return ((item.text && item.text.toLowerCase().includes(term.toLowerCase())) ||
            (item.value && item.value.toLowerCase().includes(term.toLowerCase())));
    }
}
FlexyControlSelect2Component.decorators = [
    { type: Component, args: [{
                template: `
    <ng-select
      *ngIf="!readonly && optionsData"
      #select
      [ngClass]="{ 'ng-select-multiple': multiple }"
      [formControl]="control"
      [items]="optionsData"
      [placeholder]="placeholder"
      bindLabel="text"
      bindValue="value"
      [multiple]="multiple"
      [addTag]="addItem"
      [hideSelected]="hideSelected"
      [searchFn]="enableSearchByValue ? customSearchFn : null"
      [virtualScroll]="virtualScroll"
      [loading]="loading"
      [loadingText]="loadingText"
      (change)="onChange($event)"
    >
      <ng-template ng-label-tmp let-item="item" let-clear="clear">
        <span *ngIf="item['prefixHtml']" [innerHTML]="item['prefixHtml']"></span> <span title="{{ item.text }}">{{ item.text }}</span>
        <span class="ng-value-icon right t2e-unselect-value" (click)="clear(item)">×</span>
      </ng-template>
      <ng-template ng-option-tmp let-item="item" let-index="index">
        <span *ngIf="item['prefixHtml']" [innerHTML]="item['prefixHtml']"></span> <span title="{{ item.text }}">{{ item.text }}</span>
      </ng-template>
    </ng-select>
    <flexy-control-readonly *ngIf="readonly" [value]="getReadonlyInfo(control?.value)" [default]="default"> </flexy-control-readonly>
  `,
                selector: 'flexy-control-select2',
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
FlexyControlSelect2Component.propDecorators = {
    control: [{ type: Input }],
    options: [{ type: Input }],
    default: [{ type: Input }],
    readonly: [{ type: Input }],
    multiple: [{ type: Input }],
    addItem: [{ type: Input }],
    placeholder: [{ type: Input }],
    hideSelected: [{ type: Input }],
    enableSearchByValue: [{ type: Input }],
    loading: [{ type: Input }],
    loadingText: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlSelect2Component.prototype.control;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.options;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.default;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.readonly;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.multiple;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.addItem;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.placeholder;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.hideSelected;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.enableSearchByValue;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.loading;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.loadingText;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.focused;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.clicked;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.changed;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.virtualScroll;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.optionsData;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlTagsComponent {
    constructor() {
        this.focused = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.readonlyValue = this.control.value ? this.control.value.join(', ') : this.default ? this.default : null;
    }
}
FlexyControlTagsComponent.decorators = [
    { type: Component, args: [{
                template: `
    <ng-select
      *ngIf="!readonly"
      [formControl]="control"
      [items]="options"
      [placeholder]="placeholder"
      [addTag]="true"
      [multiple]="true"
      (change)="changed.emit($event)"
      (focus)="focused.emit($event)"
    >
    </ng-select>
    <flexy-control-readonly *ngIf="readonly" [value]="readonlyValue" [default]="default"></flexy-control-readonly>
  `,
                selector: 'flexy-control-tags'
            }] }
];
FlexyControlTagsComponent.propDecorators = {
    control: [{ type: Input }],
    options: [{ type: Input }],
    default: [{ type: Input }],
    readonly: [{ type: Input }],
    placeholder: [{ type: Input }],
    focused: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlTagsComponent.prototype.control;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.options;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.default;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.focused;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.changed;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.readonlyValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlTextComponent {
    constructor() {
        this.autocomplete = '';
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
        this.entered = new EventEmitter();
    }
}
FlexyControlTextComponent.decorators = [
    { type: Component, args: [{
                template: `
    <input
      *ngIf="!readonly"
      class="form-control"
      type="text"
      [formControl]="control"
      [attr.minLength]="minLength"
      [attr.maxLength]="maxLength"
      [attr.placeholder]="default"
      [attr.autocomplete]="autocomplete"
      (focus)="focused.emit($event)"
      (click)="clicked.emit($event)"
      (change)="changed.emit($event)"
      (keyup.enter)="$event.preventDefault(); entered.emit($event)"
    />
    <flexy-control-readonly *ngIf="readonly" [value]="control.value" [default]="default"></flexy-control-readonly>
  `,
                selector: 'flexy-control-text'
            }] }
];
FlexyControlTextComponent.propDecorators = {
    control: [{ type: Input }],
    default: [{ type: Input }],
    minLength: [{ type: Input }],
    maxLength: [{ type: Input }],
    readonly: [{ type: Input }],
    autocomplete: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }],
    entered: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlTextComponent.prototype.control;
    /** @type {?} */
    FlexyControlTextComponent.prototype.default;
    /** @type {?} */
    FlexyControlTextComponent.prototype.minLength;
    /** @type {?} */
    FlexyControlTextComponent.prototype.maxLength;
    /** @type {?} */
    FlexyControlTextComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlTextComponent.prototype.autocomplete;
    /** @type {?} */
    FlexyControlTextComponent.prototype.focused;
    /** @type {?} */
    FlexyControlTextComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlTextComponent.prototype.changed;
    /** @type {?} */
    FlexyControlTextComponent.prototype.entered;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlTreeSelectComponent {
    constructor() {
        // TODO
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @param {?} event
     * @return {?}
     */
    selectItem(event) {
        this.control.setValue(event && event.id ? event.id : null);
    }
}
FlexyControlTreeSelectComponent.decorators = [
    { type: Component, args: [{
                template: `
    <div *ngIf="!readonly">
      <flexy-tree
        [tree]="dataTree"
        [selectedId]="control.value"
        [readonlyId]="readonlyId"
        [templateRef]="treeRef"
        (select)="selectItem($event)"
      >
      </flexy-tree>
      <ng-template #treeRef let-item>{{ item[labelKey] }}</ng-template>
    </div>
    <flexy-control-readonly *ngIf="readonly" [value]="control.value" [default]="default"></flexy-control-readonly>
  `,
                selector: 'flexy-control-tree-select'
            }] }
];
FlexyControlTreeSelectComponent.propDecorators = {
    control: [{ type: Input }],
    dataTree: [{ type: Input }],
    default: [{ type: Input }],
    readonlyId: [{ type: Input }],
    labelKey: [{ type: Input }],
    readonly: [{ type: Input }],
    focused: [{ type: Output }],
    clicked: [{ type: Output }],
    changed: [{ type: Output }]
};
if (false) {
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.control;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.dataTree;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.default;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.readonlyId;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.labelKey;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.focused;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormsTextareaAutosizeDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
    }
    /**
     * @param {?} textArea
     * @return {?}
     */
    onInput(textArea) {
        this.adjust();
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        this.adjust();
    }
    /**
     * @return {?}
     */
    adjust() {
        this.element.nativeElement.style.overflow = 'hidden';
        this.element.nativeElement.style.height = 'auto';
        this.element.nativeElement.style.height = this.element.nativeElement.scrollHeight + 'px';
    }
}
FlexyFormsTextareaAutosizeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'textarea[flexyAutosize]'
            },] }
];
/** @nocollapse */
FlexyFormsTextareaAutosizeDirective.ctorParameters = () => [
    { type: ElementRef }
];
FlexyFormsTextareaAutosizeDirective.propDecorators = {
    onInput: [{ type: HostListener, args: ['input', ['$event.target'],] }]
};
if (false) {
    /** @type {?} */
    FlexyFormsTextareaAutosizeDirective.prototype.element;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyFormJsonFileComponent {
}
FlexyFormJsonFileComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-form-json-file',
                template: `
    <flexy-form-field [control]="layoutSchema.formControl" [label]="label" [description]="description">
      <label role="button">
        <flexy-control-json-file [control]="layoutSchema.formControl" [readonly]="readonly" [name]="label"></flexy-control-json-file>
      </label>
    </flexy-form-field>
  `
            }] }
];
FlexyFormJsonFileComponent.propDecorators = {
    layoutSchema: [{ type: Input }],
    label: [{ type: Input }],
    description: [{ type: Input }],
    readonly: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.label;
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.description;
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FlexyControlJsonFileComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
        try {
            this.json = this.control.value ? JSON.parse(this.control.value) : null;
        }
        catch (err) {
            console.error(err);
        }
    }
    /**
     * @param {?} json
     * @return {?}
     */
    setJson(json) {
        this.control.setValue(JSON.stringify(json));
        this.control.markAsDirty();
        this.json = json;
    }
}
FlexyControlJsonFileComponent.decorators = [
    { type: Component, args: [{
                selector: 'flexy-control-json-file',
                template: `
    <ng-container *ngIf="!readonly">
      <flexy-json-import-button class="btn btn-primary" (uploaded)="setJson($event)">Upload</flexy-json-import-button>
    </ng-container>
    <ng-container *ngIf="json">
      <button type="button" flexyJsonExport [name]="name" [json]="json" class="btn btn-link">Download file</button>
    </ng-container>
  `
            }] }
];
/** @nocollapse */
FlexyControlJsonFileComponent.ctorParameters = () => [];
FlexyControlJsonFileComponent.propDecorators = {
    control: [{ type: Input }],
    readonly: [{ type: Input }],
    name: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.control;
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.name;
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.json;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const FLEXY_FORM_CONTROLS_JSON_MAPPER = {
    number: FlexyFormNumberComponent,
    text: FlexyFormTextComponent,
    select: FlexyFormSelectComponent,
    textarea: FlexyFormTextareaComponent,
    checkbox: FlexyFormCheckboxComponent,
    array: FlexyFormArrayComponent,
    group: FlexyFormGroupComponent,
    fieldset: FlexyFormFieldsetComponent,
    select2: FlexyFormSelect2Component,
    datepicker: FlexyFormDatepickerComponent,
    daterangepicker: FlexyFormDaterangepickerComponent,
    radiolist: FlexyFormRadioListComponent,
    colorpicker: FlexyFormColorpickerComponent,
    tags: FlexyFormTagsComponent,
    checkboxlist: FlexyFormCheckboxListComponent,
    treeselect: FlexyFormTreeSelectComponent,
    label: FlexyFormLabelComponent,
    tabs: FlexyFormTabsComponent,
    percent: FlexyFormPercentComponent,
    password: FlexyFormPasswordComponent,
    chips: FlexyFormChipsComponent,
    draggableselect: FlexyFormDraggableSelectComponent,
    jsonfile: FlexyFormJsonFileComponent
};
/** @type {?} */
const ENTRY_LAYOUT_COMPONENTS = [
    FlexyFormNumberComponent,
    FlexyFormSelectComponent,
    FlexyFormTextComponent,
    FlexyFormTextareaComponent,
    FlexyFormCheckboxComponent,
    FlexyFormArrayComponent,
    FlexyFormGroupComponent,
    FlexyFormFieldsetComponent,
    FlexyFormSelect2Component,
    FlexyFormDatepickerComponent,
    FlexyFormDaterangepickerComponent,
    FlexyFormRadioListComponent,
    FlexyFormColorpickerComponent,
    FlexyFormTagsComponent,
    FlexyFormCheckboxListComponent,
    FlexyFormTreeSelectComponent,
    FlexyFormLabelComponent,
    FlexyFormTabsComponent,
    FlexyFormPercentComponent,
    FlexyFormPasswordComponent,
    FlexyFormChipsComponent,
    FlexyFormDraggableSelectComponent,
    FlexyFormJsonFileComponent
];
/** @type {?} */
const FORM_CONTROLS = [
    FlexyControlCheckboxComponent,
    FlexyControlNumberComponent,
    FlexyControlSelectComponent,
    FlexyControlReadonlyComponent,
    FlexyControlTextComponent,
    FlexyControlSelect2Component,
    FlexyControlDatepickerComponent,
    FlexyControlDaterangepickerComponent,
    FlexyControlRadioListComponent,
    FlexyControlColorpickerComponent,
    FlexyControlTagsComponent,
    FlexyControlCheckboxListComponent,
    FlexyControlTreeSelectComponent,
    FlexyControlPercentComponent,
    FlexyControlPasswordComponent,
    FlexyControlChipsComponent,
    FlexyControlDraggableSelectComponent,
    FlexyControlJsonFileComponent
];
/** @type {?} */
const PUBLIC_COMPONENTS = [FlexyFormsTextareaAutosizeDirective, FlexyFieldControlInfoComponent, FlexyFieldComponent, FlexyTreeComponent];
/** @type {?} */
const COMPONENTS = [FlexyTreeNodeComponent];
class FlexyFormsBootstrapModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: FlexyFormsBootstrapModule,
            providers: [FlexyTreeService]
        };
    }
}
FlexyFormsBootstrapModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                    ReactiveFormsModule,
                    TooltipModule,
                    TabsModule,
                    TranslateModule,
                    FlexyLayoutModule,
                    FlexyFormsModule,
                    NgSelectModule,
                    TagInputModule,
                    BsDatepickerModule,
                    FlexyJsonImpExpModule,
                    DragDropModule
                ],
                declarations: [...COMPONENTS, ...PUBLIC_COMPONENTS, ...ENTRY_LAYOUT_COMPONENTS, ...FORM_CONTROLS],
                entryComponents: ENTRY_LAYOUT_COMPONENTS,
                exports: [...PUBLIC_COMPONENTS, ...ENTRY_LAYOUT_COMPONENTS, ...FORM_CONTROLS]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FLEXY_FORM_CONTROLS_JSON_MAPPER, FlexyControlCheckboxComponent, FlexyControlCheckboxListComponent, FlexyControlChipsComponent, FlexyControlColorpickerComponent, FlexyControlDatepickerComponent, FlexyControlDaterangepickerComponent, FlexyControlDraggableSelectComponent, FlexyControlNumberComponent, FlexyControlPasswordComponent, FlexyControlPercentComponent, FlexyControlRadioListComponent, FlexyControlReadonlyComponent, FlexyControlSelect2Component, FlexyControlSelectComponent, FlexyControlTagsComponent, FlexyControlTextComponent, FlexyControlTreeSelectComponent, FlexyFieldComponent, FlexyFieldControlInfoComponent, FlexyFormArrayComponent, FlexyFormCheckboxComponent, FlexyFormCheckboxListComponent, FlexyFormChipsComponent, FlexyFormColorpickerComponent, FlexyFormDatepickerComponent, FlexyFormDaterangepickerComponent, FlexyFormDraggableSelectComponent, FlexyFormFieldsetComponent, FlexyFormGroupComponent, FlexyFormLabelComponent, FlexyFormNumberComponent, FlexyFormPasswordComponent, FlexyFormPercentComponent, FlexyFormRadioListComponent, FlexyFormSelect2Component, FlexyFormSelectComponent, FlexyFormTabsComponent, FlexyFormTagsComponent, FlexyFormTextComponent, FlexyFormTextareaComponent, FlexyFormTreeSelectComponent, FlexyFormsBootstrapModule, FlexyFormsTextareaAutosizeDirective, FlexyTreeComponent, FlexyTreeNodeComponent, FlexyTreeService, FlexyFormJsonFileComponent as ɵa, FlexyControlJsonFileComponent as ɵb };
//# sourceMappingURL=ng-flexy-form-bootstrap.js.map
