import { Component, ChangeDetectorRef, ElementRef, Input, ChangeDetectionStrategy, ViewChild, ApplicationRef, Injectable, EventEmitter, Output, Directive, HostListener, NgModule } from '@angular/core';
import { TranslateService, TranslateModule } from '@ngx-translate/core';
import { isRequired, FlexyFormJsonMapperService, FlexyFormSchemaService, FlexyFormsModule } from '@ng-flexy/form';
import { FlexyToastsService } from '@ng-flexy/toasts';
import { FlexyLoggerService } from '@ng-flexy/core';
import { moveItemInArray, DragDropModule } from '@angular/cdk/drag-drop';
import { FormControl, Validators, FormBuilder, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { HttpClient } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { template, get, cloneDeep } from 'lodash';
import { BehaviorSubject } from 'rxjs';
import * as momentNs from 'moment';
import { __spread } from 'tslib';
import { CommonModule } from '@angular/common';
import { FlexyLayoutModule } from '@ng-flexy/layout';
import { TooltipModule, TabsModule, BsDatepickerModule } from 'ngx-bootstrap';
import { NgSelectModule } from '@ng-select/ng-select';
import { TagInputModule } from 'ngx-chips';
import { FlexyJsonImpExpModule } from '@ng-flexy/json-impexp';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormArrayComponent = /** @class */ (function () {
    function FlexyFormArrayComponent(jsonMapperService, toasts, changeDetectorRef, translate, logger, elementRef) {
        this.jsonMapperService = jsonMapperService;
        this.toasts = toasts;
        this.changeDetectorRef = changeDetectorRef;
        this.translate = translate;
        this.logger = logger;
        this.elementRef = elementRef;
        this.addable = true;
        this.draggable = false;
        this.removable = true;
        this.removeAny = false;
        this.showAddButton = false;
        this.showRemoveButton = false;
    }
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.enableButtons();
        if (this.layoutSchema.formControl) {
            this.isRequired = isRequired(this.layoutSchema.formControl);
        }
        if (this.jsonSchema &&
            this.jsonSchema.validators.minItems &&
            (!this.layoutSchema.items || this.layoutSchema.items.length < this.jsonSchema.validators.minItems)) {
            if (!this.readonly) {
                /** @type {?} */
                var minLength = Math.max(this.layoutSchema.items && this.layoutSchema.items.length ? this.layoutSchema.items.length : 0, this.jsonSchema.validators.minItems);
                for (var i = 0; i < minLength; i++) {
                    this.addNew();
                }
            }
        }
    };
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.addNew = /**
     * @return {?}
     */
    function () {
        if (this.maxItemsExceeded()) {
            this.toasts.error(this.translate.instant('FLEXY_FORM_ARRAY_MAX_VALIDATOR_ERROR', { max: this.jsonSchema.validators.maxItems }));
        }
        else {
            /** @type {?} */
            var newValue = this.jsonSchema.items.children ? {} : null;
            /** @type {?} */
            var control = this.jsonMapperService.createItemControl(this.jsonSchema.items, this.readonly, newValue);
            ((/** @type {?} */ (this.layoutSchema.formControl))).push(control);
            this.layoutSchema.items.push(this.jsonMapperService.createArrayItemSchema(control, this.jsonSchema.items, this.jsonSchema.itemIndexDef, null, this.readonly, {}, newValue, this.layoutSchema.items.length, this.layoutSchema.formControl));
            this.enableButtons();
        }
    };
    /**
     * @param {?} index
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.removeItem = /**
     * @param {?} index
     * @return {?}
     */
    function (index) {
        var _this = this;
        this.toasts.confirm(this.translate.instant('FLEXY_FORM_DELETE_ITEM_CONFIRM'), '', (/**
         * @return {?}
         */
        function () {
            _this.layoutSchema.items.splice(index, 1);
            ((/** @type {?} */ (_this.layoutSchema.formControl))).removeAt(index);
            _this.layoutSchema.formControl.markAsDirty();
        }));
    };
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.removeLast = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.toasts.confirm(this.translate.instant('FLEXY_FORM_DELETE_LAST_ITEM_CONFIRM'), '', (/**
         * @return {?}
         */
        function () {
            _this.removeLastItem();
        }));
    };
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.removeLastItem = /**
     * @return {?}
     */
    function () {
        if (this.layoutSchema.items.length > 0) {
            /** @type {?} */
            var inx = this.layoutSchema.items.length - 1;
            this.layoutSchema.items.splice(inx, 1);
            ((/** @type {?} */ (this.layoutSchema.formControl))).removeAt(inx);
            ((/** @type {?} */ (this.layoutSchema.formControl))).markAsDirty();
            this.enableButtons();
        }
    };
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.maxItemsExceeded = /**
     * @return {?}
     */
    function () {
        return this.jsonSchema && this.jsonSchema.validators && this.layoutSchema.items.length >= this.jsonSchema.validators.maxItems;
    };
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.minItemsExceeded = /**
     * @return {?}
     */
    function () {
        return (this.jsonSchema &&
            this.jsonSchema.validators &&
            (!this.layoutSchema.items.length || this.layoutSchema.items.length <= this.jsonSchema.validators.minItems));
    };
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.focusOnAdded = /**
     * @return {?}
     */
    function () {
        this.changeDetectorRef.detectChanges();
        /** @type {?} */
        var added = this.elementRef.nativeElement.querySelector('.bottom-button');
        if (added) {
            added.focus();
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.drop = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        moveItemInArray(this.layoutSchema.items, event.previousIndex, event.currentIndex);
        this.layoutSchema.formControl.markAsDirty();
    };
    /**
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.isDraggable = /**
     * @return {?}
     */
    function () {
        return !this.readonly && this.draggable && this.layoutSchema.items.length > 1;
    };
    /**
     * @private
     * @return {?}
     */
    FlexyFormArrayComponent.prototype.enableButtons = /**
     * @private
     * @return {?}
     */
    function () {
        this.showAddButton = !this.readonly && !this.maxItemsExceeded();
        this.showRemoveButton = !this.readonly && !this.minItemsExceeded();
    };
    FlexyFormArrayComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-array',
                    template: "<fieldset>\n  <h4 *ngIf=\"legend || (!readonly && layoutSchema?.items?.length > 10)\">\n    <span *ngIf=\"legend\">\n      {{ legend }}\n      <span *ngIf=\"isRequired\" class=\"isRequired\" [tooltip]=\"'FLEXY_FORM_FIELD_IS_REQUIRED' | translate\">(<b>*</b>)</span>\n    </span>\n\n    <div class=\"actions\" *ngIf=\"!readonly && layoutSchema?.items?.length > 10\">\n      <button type=\"button\" class=\"btn btn-info btn-outline btn-sm\" [disabled]=\"!showAddButton\" (click)=\"addNew(); focusOnAdded()\">\n        <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n      </button>\n    </div>\n  </h4>\n\n  <ng-container *ngIf=\"layoutSchema && layoutSchema.items\">\n    <div cdkDropList class=\"drag-container schema-items\" [cdkDropListDisabled]=\"!isDraggable()\" (cdkDropListDropped)=\"drop($event)\">\n      <div\n        *ngFor=\"let componentSchema of layoutSchema.items; let index = index\"\n        cdkDrag\n        cdkDragLockAxis=\"y\"\n        class=\"flexy-form-array-draggable-item schema-item\"\n        cdkDragBoundary=\".drag-container\"\n      >\n        <div class=\"schema-item-body\">\n          <flexy-layout [schema]=\"[componentSchema]\" [ngClass]=\"{ 'removable-array-item': !readonly && removable && removeAny }\">\n            <button\n              *ngIf=\"!readonly && removable && removeAny\"\n              type=\"button\"\n              class=\"btn btn-outline delete-array-item e2e-btn-delete-array-item\"\n              tooltip=\"{{ 'FLEXY_FORM_ARRAY_REMOVE_ITEM' | translate }}\"\n              (click)=\"removeItem(index)\"\n            >\n              <i class=\"flexy-icon-remove\"></i>\n            </button>\n          </flexy-layout>\n        </div>\n        <div *ngIf=\"isDraggable()\" class=\"drag-handle\" cdkDragHandle>\n          <button type=\"button\" class=\"btn btn-link\" tooltip=\"{{ 'FLEXY_FORM_ARRAY_REORDER' | translate }}\">\n            <i class=\"flexy-icon-move\"></i>\n          </button>\n        </div>\n      </div>\n    </div>\n\n    <span class=\"actions\" *ngIf=\"!readonly && removable\">\n      <button\n        type=\"button\"\n        class=\"btn btn-info btn-outline btn-sm bottom-button t2e-array-add-btn\"\n        [disabled]=\"!showAddButton\"\n        (click)=\"addNew()\"\n      >\n        <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n      </button>\n      <button\n        *ngIf=\"!removeAny\"\n        type=\"button\"\n        class=\"btn btn-danger btn-outline btn-sm remove-item\"\n        [disabled]=\"!showRemoveButton\"\n        (click)=\"removeLast()\"\n      >\n        {{ 'FLEXY_FORM_ARRAY_REMOVE_LAST' | translate }}\n      </button>\n    </span>\n\n    <flexy-form-field-info *ngIf=\"layoutSchema.formControl?.validator\" [control]=\"layoutSchema.formControl\"></flexy-form-field-info>\n    <p class=\"no-data\" *ngIf=\"!layoutSchema.items.length\">\n      {{ 'FLEXY_FORM_ARRAY_NO_DATA' | translate }}\n    </p>\n  </ng-container>\n</fieldset>\n"
                }] }
    ];
    /** @nocollapse */
    FlexyFormArrayComponent.ctorParameters = function () { return [
        { type: FlexyFormJsonMapperService },
        { type: FlexyToastsService },
        { type: ChangeDetectorRef },
        { type: TranslateService },
        { type: FlexyLoggerService },
        { type: ElementRef }
    ]; };
    FlexyFormArrayComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        legend: [{ type: Input }],
        jsonSchema: [{ type: Input }],
        readonly: [{ type: Input }],
        addable: [{ type: Input }],
        draggable: [{ type: Input }],
        removable: [{ type: Input }],
        removeAny: [{ type: Input }]
    };
    return FlexyFormArrayComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormArrayComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.legend;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.jsonSchema;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.addable;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.draggable;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.removable;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.removeAny;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.isRequired;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.showAddButton;
    /** @type {?} */
    FlexyFormArrayComponent.prototype.showRemoveButton;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.jsonMapperService;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.toasts;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.changeDetectorRef;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.translate;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.logger;
    /**
     * @type {?}
     * @private
     */
    FlexyFormArrayComponent.prototype.elementRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormCheckboxComponent = /** @class */ (function () {
    function FlexyFormCheckboxComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormCheckboxComponent.prototype.toogle = /**
     * @return {?}
     */
    function () {
        if (!this.readonly) {
            this.layoutSchema.formControl.markAsDirty();
            this.layoutSchema.formControl.setValue(!this.layoutSchema.formControl.value);
        }
    };
    FlexyFormCheckboxComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-checkbox',
                    template: "\n    <flexy-form-field [control]=\"layoutSchema.formControl\" [label]=\"label\" [description]=\"description\" [ngClass]=\"{ readonly: readonly }\">\n      <label (click)=\"toogle()\" role=\"button\">\n        <flexy-control-checkbox\n          [control]=\"layoutSchema.formControl\"\n          [readonly]=\"readonly\"\n          [default]=\"default\"\n          [onlyLabel]=\"onlyLabel\"\n          [faIcon]=\"faIcon\"\n        ></flexy-control-checkbox>\n        <span *ngIf=\"suffix\"> {{ suffix }}</span>\n      </label>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormCheckboxComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        label: [{ type: Input }],
        default: [{ type: Input }],
        description: [{ type: Input }],
        readonly: [{ type: Input }],
        onlyLabel: [{ type: Input }],
        faIcon: [{ type: Input }],
        suffix: [{ type: Input }]
    };
    return FlexyFormCheckboxComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.label;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.default;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.description;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.onlyLabel;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.faIcon;
    /** @type {?} */
    FlexyFormCheckboxComponent.prototype.suffix;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormCheckboxListComponent = /** @class */ (function () {
    function FlexyFormCheckboxListComponent() {
    }
    FlexyFormCheckboxListComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-checkbox-list',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-checkbox-list\n        [name]=\"layoutSchema.formName\"\n        [options]=\"options\"\n        [readonly]=\"readonly\"\n        [default]=\"default\"\n        [control]=\"layoutSchema.formControl\"\n      >\n      </flexy-control-checkbox-list>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormCheckboxListComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        options: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormCheckboxListComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.default;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.options;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.label;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.description;
    /** @type {?} */
    FlexyFormCheckboxListComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormChipsComponent = /** @class */ (function () {
    function FlexyFormChipsComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormChipsComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.default + ')';
            }
        }
    };
    FlexyFormChipsComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-chips',
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [labelIcon]=\"labelIcon\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <div class=\"input-group\">\n        <div class=\"input-group-addon\" *ngIf=\"prefix\">{{ prefix }}</div>\n        <flexy-control-chips\n          [control]=\"layoutSchema.formControl\"\n          [default]=\"default\"\n          [readonly]=\"readonly\"\n          [autocompleteItems]=\"autocompleteItems\"\n          [placeholder]=\"placeholder\"\n          [disableDragging]=\"disableDragging\"\n          [animations]=\"animations\"\n          [onlyFromAutocomplete]=\"onlyFromAutocomplete\"\n        ></flexy-control-chips>\n        <div class=\"input-group-addon\" *ngIf=\"suffix\">{{ suffix }}</div>\n      </div>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormChipsComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        autocompleteItems: [{ type: Input }],
        placeholder: [{ type: Input }],
        labelIcon: [{ type: Input }],
        disableDragging: [{ type: Input }],
        animations: [{ type: Input }],
        onlyFromAutocomplete: [{ type: Input }],
        prefix: [{ type: Input }],
        suffix: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormChipsComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormChipsComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.default;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.label;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.description;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.autocompleteItems;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.disableDragging;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.animations;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.onlyFromAutocomplete;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormChipsComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormColorpickerComponent = /** @class */ (function () {
    function FlexyFormColorpickerComponent() {
    }
    FlexyFormColorpickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-colorpicker',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-colorpicker [readonly]=\"readonly\" [default]=\"default\" [list]=\"list\" [control]=\"layoutSchema.formControl\">\n      </flexy-control-colorpicker>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormColorpickerComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        list: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        readonly: [{ type: Input }],
        inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
    };
    return FlexyFormColorpickerComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.default;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.list;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.label;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.description;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormColorpickerComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormDatepickerComponent = /** @class */ (function () {
    function FlexyFormDatepickerComponent() {
    }
    FlexyFormDatepickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-datepicker',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-datepicker\n        [control]=\"layoutSchema.formControl\"\n        [default]=\"default\"\n        [min]=\"min\"\n        [max]=\"max\"\n        [readonly]=\"readonly\"\n        [isDisabled]=\"isDisabled\"\n        [placeholder]=\"placeholder\"\n        [theme]=\"theme\"\n        [format]=\"format\"\n        [hideButton]=\"hideButton\"\n        [minMode]=\"minMode\"\n      >\n      </flexy-control-datepicker>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormDatepickerComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        placeholder: [{ type: Input }],
        theme: [{ type: Input }],
        readonly: [{ type: Input }],
        isDisabled: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        format: [{ type: Input }],
        hideButton: [{ type: Input }],
        minMode: [{ type: Input }]
    };
    return FlexyFormDatepickerComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.default;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.min;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.max;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.label;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.description;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.format;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.hideButton;
    /** @type {?} */
    FlexyFormDatepickerComponent.prototype.minMode;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormDaterangepickerComponent = /** @class */ (function () {
    function FlexyFormDaterangepickerComponent() {
    }
    FlexyFormDaterangepickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-daterangepicker',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-daterangepicker\n        [control]=\"layoutSchema.formControl\"\n        [default]=\"default\"\n        [min]=\"min\"\n        [max]=\"max\"\n        [readonly]=\"readonly\"\n        [isDisabled]=\"isDisabled\"\n        [placeholder]=\"placeholder\"\n        [theme]=\"theme\"\n        [format]=\"format\"\n        [customClass]=\"customClass\"\n      >\n      </flexy-control-daterangepicker>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormDaterangepickerComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        placeholder: [{ type: Input }],
        theme: [{ type: Input }],
        readonly: [{ type: Input }],
        isDisabled: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        format: [{ type: Input }],
        customClass: [{ type: Input }]
    };
    return FlexyFormDaterangepickerComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.default;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.min;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.max;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.label;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.description;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.format;
    /** @type {?} */
    FlexyFormDaterangepickerComponent.prototype.customClass;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormDraggableSelectComponent = /** @class */ (function () {
    function FlexyFormDraggableSelectComponent() {
    }
    FlexyFormDraggableSelectComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-draggable-select',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [labelIcon]=\"labelIcon\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-draggable-select\n        [control]=\"layoutSchema.formControl\"\n        [default]=\"default\"\n        [readonly]=\"readonly\"\n        [options]=\"options\"\n        [placeholder]=\"placeholder\"\n        [hideSelected]=\"hideSelected\"\n        [enableSearchByValue]=\"enableSearchByValue\"\n      >\n      </flexy-control-draggable-select>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormDraggableSelectComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        options: [{ type: Input }],
        placeholder: [{ type: Input }],
        labelIcon: [{ type: Input }],
        readonly: [{ type: Input }],
        hideSelected: [{ type: Input }],
        enableSearchByValue: [{ type: Input }]
    };
    return FlexyFormDraggableSelectComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.default;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.label;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.description;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.options;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.hideSelected;
    /** @type {?} */
    FlexyFormDraggableSelectComponent.prototype.enableSearchByValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFieldComponent = /** @class */ (function () {
    function FlexyFieldComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFieldComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (this.control) {
            this.isRequired = isRequired(this.control);
        }
    };
    /**
     * @return {?}
     */
    FlexyFieldComponent.prototype.focusControl = /**
     * @return {?}
     */
    function () {
        if (this.control && this.fieldContainerRef && this.fieldContainerRef.nativeElement) {
            /** @type {?} */
            var inputs = this.fieldContainerRef.nativeElement.getElementsByTagName('input');
            if (!inputs[0]) {
                inputs = this.fieldContainerRef.nativeElement.getElementsByTagName('textarea');
            }
            if (inputs[0]) {
                ((/** @type {?} */ (inputs[0]))).focus();
            }
        }
    };
    FlexyFieldComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-field',
                    template: "\n    <div\n      class=\"flexy-field\"\n      [ngClass]=\"{\n        'has-error': control?.dirty && !control.pending && !control.valid,\n        'has-success': control?.dirty && !(!control.pending && !control.valid),\n        'is-required': isRequired\n      }\"\n      #fieldContainerRef\n    >\n      <label\n        *ngIf=\"label\"\n        class=\"flexy-field-label\"\n        [ngClass]=\"{ 'not-empty': control.value || control.value === 0 }\"\n        (click)=\"focusControl(); $event.stopPropagation()\"\n        [attr.data-rel-name]=\"name\"\n      >\n        <i *ngIf=\"labelIcon\" [class]=\"labelIcon\"></i> {{ label | translate }}{{ label[label.length - 1] !== ':' ? ':' : '' }}\n        <span *ngIf=\"isRequired\" class=\"isRequired\" [tooltip]=\"'FLEXY_FORM_FIELD_IS_REQUIRED' | translate\">(<b>*</b>)</span>\n      </label>\n      <label *ngIf=\"!label && isRequired\" class=\"isRequired\" [tooltip]=\"'FLEXY_FORM_FIELD_IS_REQUIRED' | translate\">(<b>*</b>)</label>\n\n      <div class=\"flexy-field-control\">\n        <ng-content></ng-content>\n        <flexy-form-field-info *ngIf=\"control\" [control]=\"control\" [description]=\"description\"></flexy-form-field-info>\n      </div>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyFieldComponent.ctorParameters = function () { return []; };
    FlexyFieldComponent.propDecorators = {
        control: [{ type: Input }],
        name: [{ type: Input }],
        label: [{ type: Input }],
        labelIcon: [{ type: Input }],
        description: [{ type: Input }],
        fieldContainerRef: [{ type: ViewChild, args: ['fieldContainerRef', { static: false },] }]
    };
    return FlexyFieldComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFieldComponent.prototype.control;
    /** @type {?} */
    FlexyFieldComponent.prototype.name;
    /** @type {?} */
    FlexyFieldComponent.prototype.label;
    /** @type {?} */
    FlexyFieldComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFieldComponent.prototype.description;
    /** @type {?} */
    FlexyFieldComponent.prototype.fieldContainerRef;
    /** @type {?} */
    FlexyFieldComponent.prototype.isRequired;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFieldControlInfoComponent = /** @class */ (function () {
    function FlexyFieldControlInfoComponent() {
        this.forceDirty = false;
    }
    FlexyFieldControlInfoComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-field-info',
                    template: "\n    <div *ngIf=\"control && (forceDirty || (control.dirty && !control.pending)) && !control.valid\" class=\"field-info-error\">\n      <ng-container *ngIf=\"control?.errors\">\n        <small class=\"form-text text-danger\">\n          {{ control.errors | firstError }}\n        </small>\n      </ng-container>\n    </div>\n    <div *ngIf=\"!(control && (forceDirty || (control.dirty && !control.pending)) && !control.valid)\" class=\"field-info-description\">\n      <small class=\"form-text text-muted\" [innerHtml]=\"description\"></small>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyFieldControlInfoComponent.ctorParameters = function () { return []; };
    FlexyFieldControlInfoComponent.propDecorators = {
        control: [{ type: Input }],
        description: [{ type: Input }],
        forceDirty: [{ type: Input }]
    };
    return FlexyFieldControlInfoComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFieldControlInfoComponent.prototype.control;
    /** @type {?} */
    FlexyFieldControlInfoComponent.prototype.description;
    /** @type {?} */
    FlexyFieldControlInfoComponent.prototype.forceDirty;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormFieldsetComponent = /** @class */ (function () {
    function FlexyFormFieldsetComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormFieldsetComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    FlexyFormFieldsetComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-fieldset',
                    template: "<fieldset>\n  <h4>\n    <span *ngIf=\"legend\">{{ legend }}</span>\n  </h4>\n\n  <flexy-layout [schema]=\"layoutSchema.children\"></flexy-layout>\n  <flexy-form-field-info *ngIf=\"layoutSchema.formControl?.validator\" [control]=\"layoutSchema.formControl\"></flexy-form-field-info>\n</fieldset>\n"
                }] }
    ];
    /** @nocollapse */
    FlexyFormFieldsetComponent.ctorParameters = function () { return []; };
    FlexyFormFieldsetComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        legend: [{ type: Input }]
    };
    return FlexyFormFieldsetComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormFieldsetComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormFieldsetComponent.prototype.legend;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormGroupComponent = /** @class */ (function () {
    function FlexyFormGroupComponent(jsonMapperService, schemaService, toasts, translate, changeDetectorRef, elementRef) {
        this.jsonMapperService = jsonMapperService;
        this.schemaService = schemaService;
        this.toasts = toasts;
        this.translate = translate;
        this.changeDetectorRef = changeDetectorRef;
        this.elementRef = elementRef;
        this.addable = true;
        this.removable = true;
        this.isRequired = false;
        this.isAddFormVisible = false;
    }
    /**
     * @return {?}
     */
    FlexyFormGroupComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (this.layoutSchema && !this.layoutSchema.children) {
            this.layoutSchema.children = [];
        }
        this.isRequired = isRequired(this.layoutSchema.formControl);
        this.addFormControl = new FormControl(this.prepareNewKey(), [
            Validators.required,
            this.keyValidator(this.jsonSchema, this.layoutSchema)
        ]);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyFormGroupComponent.prototype.addNew = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        var _a;
        event.stopPropagation();
        event.preventDefault();
        if (this.addFormControl.valid) {
            /** @type {?} */
            var key = this.addFormControl.value;
            this.addFormControl.reset();
            if (this.schemaService.addGroupItemToSchema(this.layoutSchema, key, this.jsonSchema, this.parentGroupName + '.' + key, this.readonly)) {
                /** @type {?} */
                var translateKey = 'key';
                this.toasts.success(this.translate.instant('FLEXY_FORM_GROUP_ADD_ITEM_SUCCESS', (_a = {}, _a[translateKey] = key, _a)));
                this.addFormControl.setValue(this.prepareNewKey());
                this.isAddFormVisible = false;
            }
        }
    };
    /**
     * @param {?} index
     * @return {?}
     */
    FlexyFormGroupComponent.prototype.removeItem = /**
     * @param {?} index
     * @return {?}
     */
    function (index) {
        var _this = this;
        this.toasts.confirm(this.translate.instant('FLEXY_FORM_DELETE_ITEM_CONFIRM'), '', (/**
         * @return {?}
         */
        function () {
            _this.schemaService.removeGroupItemToSchema(_this.layoutSchema, index);
        }));
    };
    /**
     * @return {?}
     */
    FlexyFormGroupComponent.prototype.focusOnNewInput = /**
     * @return {?}
     */
    function () {
        this.changeDetectorRef.detectChanges();
        /** @type {?} */
        var input = this.elementRef.nativeElement.querySelector('.key-control-text input');
        if (input) {
            input.focus();
        }
    };
    /**
     * @private
     * @return {?}
     */
    FlexyFormGroupComponent.prototype.prepareNewKey = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.jsonSchema.itemKeyGen) {
            return this._randomKey(this.jsonSchema.itemKeyGen);
        }
        else {
            return void 0;
        }
    };
    /**
     * schema is string /(([a-zA-Z0-9]*){([sd])\.?([0-9]*)}+([a-zA-Z0-9]*))/gys
     * supported {s}: string {d}: number with defined size (default :8 max 32) {s.2}
     *
     * for example:
     *
     * AA{d.2}BB{s}
     *
     * generate: AA23BBasewASDq
     */
    /**
     * schema is string /(([a-zA-Z0-9]*){([sd])\.?([0-9]*)}+([a-zA-Z0-9]*))/gys
     * supported {s}: string {d}: number with defined size (default :8 max 32) {s.2}
     *
     * for example:
     *
     * AA{d.2}BB{s}
     *
     * generate: AA23BBasewASDq
     * @private
     * @param {?} schema
     * @return {?}
     */
    FlexyFormGroupComponent.prototype._randomKey = /**
     * schema is string /(([a-zA-Z0-9]*){([sd])\.?([0-9]*)}+([a-zA-Z0-9]*))/gys
     * supported {s}: string {d}: number with defined size (default :8 max 32) {s.2}
     *
     * for example:
     *
     * AA{d.2}BB{s}
     *
     * generate: AA23BBasewASDq
     * @private
     * @param {?} schema
     * @return {?}
     */
    function (schema) {
        /** @type {?} */
        var newKey = '';
        /** @type {?} */
        var re = /(([a-zA-Z0-9]*){([sd])\.?([0-9]*)}+([a-zA-Z0-9]*))/gy;
        /** @type {?} */
        var xArray = re.exec(schema);
        while (xArray) {
            newKey += xArray[2] + this._generateRandom(xArray[3], parseInt(xArray[4], 10)) + xArray[5];
            xArray = re.exec(schema);
        }
        return newKey;
    };
    /**
     * @private
     * @param {?} type
     * @param {?} length
     * @return {?}
     */
    FlexyFormGroupComponent.prototype._generateRandom = /**
     * @private
     * @param {?} type
     * @param {?} length
     * @return {?}
     */
    function (type, length) {
        if (!length) {
            length = 2;
        }
        length = Math.min(length, 32);
        /** @type {?} */
        var shuffleSource = type === 'd' ? '12345678901234567890123456789012345678901234567890' : 'abcdefghijklmnoprqstwxzABCDEFGHIJKLMNOPRQSTWXZ';
        /** @type {?} */
        var s = this._shuffle(shuffleSource.split(''));
        if (length) {
            s.length = length;
        }
        return s.join('');
    };
    /**
     * @private
     * @param {?} array
     * @return {?}
     */
    FlexyFormGroupComponent.prototype._shuffle = /**
     * @private
     * @param {?} array
     * @return {?}
     */
    function (array) {
        /** @type {?} */
        var counter = array.length;
        // While there are elements in the array
        while (counter > 0) {
            // Pick a random index
            /** @type {?} */
            var index = Math.floor(Math.random() * counter);
            // Decrease counter by 1
            counter--;
            // And swap the last element with it
            /** @type {?} */
            var temp = array[counter];
            array[counter] = array[index];
            array[index] = temp;
        }
        return array;
    };
    // private validateKey(): string {
    //   const err = this.schemaService.validateGroupKey(this.defineKey, this.jsonSchema);
    //   if (err) {
    //     return err;
    //   } else if (Object.keys((<FormGroup>this.layoutSchema.formControl).controls).indexOf(this.defineKey) !== -1) {
    //     return 'exist';
    //   }
    //   return null;
    // }
    // private validateKey(): string {
    //   const err = this.schemaService.validateGroupKey(this.defineKey, this.jsonSchema);
    //   if (err) {
    //     return err;
    //   } else if (Object.keys((<FormGroup>this.layoutSchema.formControl).controls).indexOf(this.defineKey) !== -1) {
    //     return 'exist';
    //   }
    //   return null;
    // }
    /**
     * @private
     * @param {?} jsonSchema
     * @param {?} layoutSchema
     * @return {?}
     */
    FlexyFormGroupComponent.prototype.keyValidator = 
    // private validateKey(): string {
    //   const err = this.schemaService.validateGroupKey(this.defineKey, this.jsonSchema);
    //   if (err) {
    //     return err;
    //   } else if (Object.keys((<FormGroup>this.layoutSchema.formControl).controls).indexOf(this.defineKey) !== -1) {
    //     return 'exist';
    //   }
    //   return null;
    // }
    /**
     * @private
     * @param {?} jsonSchema
     * @param {?} layoutSchema
     * @return {?}
     */
    function (jsonSchema, layoutSchema) {
        return (/**
         * @param {?} control
         * @return {?}
         */
        function (control) {
            /** @type {?} */
            var patternReg = jsonSchema.itemKeyPattern ? new RegExp(jsonSchema.itemKeyPattern, 'g') : null;
            if (patternReg && !patternReg.exec(control.value)) {
                return { key_wrong_format: { key: control.value, pattern: patternReg } };
            }
            else if (Object.keys(((/** @type {?} */ (layoutSchema.formControl))).controls).indexOf(control.value) !== -1) {
                return { key_exist: { key: control.value } };
            }
            return null;
        });
    };
    FlexyFormGroupComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-group',
                    template: "<fieldset>\n  <h4>\n    <span *ngIf=\"legend\">\n      {{ legend }}\n      <span *ngIf=\"isRequired\" class=\"isRequired\" [tooltip]=\"'FLEXY_FORM_FIELD_IS_REQUIRED' | translate\">(<b>*</b>)</span>\n    </span>\n    <div class=\"actions\" *ngIf=\"!readonly && addable && layoutSchema?.children?.length > 10\">\n      <button type=\"button\" class=\"btn btn-info btn-outline btn-sm\" (click)=\"isAddFormVisible = true; focusOnNewInput()\">\n        <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n      </button>\n    </div>\n  </h4>\n\n  <ng-container *ngIf=\"layoutSchema && layoutSchema.children\">\n    <ng-container *ngFor=\"let item of layoutSchema.children; let index = index\">\n      <flexy-layout [schema]=\"[item]\">\n        <button\n          *ngIf=\"!readonly && removable\"\n          type=\"button\"\n          class=\"btn btn-outline delete-item e2e-btn-delete-group-item\"\n          (click)=\"removeItem(index)\"\n        >\n          <i class=\"flexy-icon-remove\"></i>\n        </button>\n      </flexy-layout>\n    </ng-container>\n  </ng-container>\n\n  <div class=\"actions add-form-control\" *ngIf=\"!readonly && addable\">\n    <button\n      type=\"button\"\n      *ngIf=\"!isAddFormVisible\"\n      class=\"btn btn-info btn-outline btn-sm\"\n      (click)=\"isAddFormVisible = true; focusOnNewInput()\"\n    >\n      <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n    </button>\n    <flexy-form-field *ngIf=\"isAddFormVisible\" class=\"input-group\" [control]=\"addFormControl\">\n      <div class=\"input-group\">\n        <flexy-control-text class=\"key-control-text\" [control]=\"addFormControl\" (entered)=\"addNew($event)\"></flexy-control-text>\n        <span class=\"input-group-btn input-group-append\">\n          <button\n            type=\"button\"\n            [disabled]=\"!addFormControl.valid\"\n            class=\"btn btn-info btn-outline btn-sm add-item\"\n            (click)=\"addNew($event)\"\n          >\n            {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n          </button>\n        </span>\n        <span class=\"input-group-btn input-group-append\">\n          <button type=\"button\" class=\"btn btn-link btn-sm\" (click)=\"isAddFormVisible = false\">\n            {{ 'FLEXY_FORM_ARRAY_ADD_CANCEL' | translate }}\n          </button>\n        </span>\n      </div>\n    </flexy-form-field>\n  </div>\n\n  <flexy-form-field-info\n    *ngIf=\"layoutSchema && layoutSchema.formControl\"\n    [control]=\"layoutSchema.formControl\"\n    [forceDirty]=\"true\"\n  ></flexy-form-field-info>\n\n  <p class=\"no-data\" *ngIf=\"layoutSchema && !layoutSchema.children.length\">\n    {{ 'FLEXY_FORM_ARRAY_NO_DATA' | translate }}\n  </p>\n</fieldset>\n"
                }] }
    ];
    /** @nocollapse */
    FlexyFormGroupComponent.ctorParameters = function () { return [
        { type: FlexyFormJsonMapperService },
        { type: FlexyFormSchemaService },
        { type: FlexyToastsService },
        { type: TranslateService },
        { type: ChangeDetectorRef },
        { type: ElementRef }
    ]; };
    FlexyFormGroupComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        legend: [{ type: Input }],
        readonly: [{ type: Input }],
        addable: [{ type: Input }],
        removable: [{ type: Input }],
        jsonSchema: [{ type: Input }],
        parentGroupName: [{ type: Input }]
    };
    return FlexyFormGroupComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormGroupComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.legend;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.addable;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.removable;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.jsonSchema;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.parentGroupName;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.isRequired;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.isAddFormVisible;
    /** @type {?} */
    FlexyFormGroupComponent.prototype.addFormControl;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.jsonMapperService;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.schemaService;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.toasts;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.translate;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.changeDetectorRef;
    /**
     * @type {?}
     * @private
     */
    FlexyFormGroupComponent.prototype.elementRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormLabelComponent = /** @class */ (function () {
    function FlexyFormLabelComponent() {
    }
    FlexyFormLabelComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-label',
                    template: "\n    <label>{{ label }}</label>\n  "
                }] }
    ];
    FlexyFormLabelComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        label: [{ type: Input }]
    };
    return FlexyFormLabelComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormLabelComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormLabelComponent.prototype.label;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormNumberComponent = /** @class */ (function () {
    function FlexyFormNumberComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormNumberComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (this.refPath && this.refPath.length) {
            /** @type {?} */
            var refControl = this.getRefControl(this.refPath, this.layoutSchema.formControl);
            this.refControl = refControl;
        }
    };
    /**
     * @private
     * @param {?} path
     * @param {?} node
     * @return {?}
     */
    FlexyFormNumberComponent.prototype.getRefControl = /**
     * @private
     * @param {?} path
     * @param {?} node
     * @return {?}
     */
    function (path, node) {
        // tslint:disable-next-line:prefer-for-of
        for (var i = 0; i < path.length; i++) {
            node = node.parent;
        }
        // tslint:disable-next-line:prefer-for-of
        for (var i = 0; i < path.length; i++) {
            node = Number.isInteger(path[i]) ? node.get(Object.keys(node.controls)[path[i]]) : node.controls[path[i]];
        }
        return node;
    };
    FlexyFormNumberComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-number',
                    template: "\n    <flexy-form-field\n      *ngIf=\"!refPath || refControl\"\n      [control]=\"refControl ? refControl : layoutSchema.formControl\"\n      [label]=\"label\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <div class=\"addon\" *ngIf=\"prefix\">{{ prefix }}</div>\n      <flexy-control-number\n        [control]=\"refControl ? refControl : layoutSchema.formControl\"\n        [min]=\"min\"\n        [max]=\"max\"\n        [step]=\"step\"\n        [default]=\"default\"\n        [decimal]=\"decimal\"\n        [readonly]=\"readonly\"\n      >\n      </flexy-control-number>\n      <div class=\"addon\" *ngIf=\"suffix\">{{ suffix }}</div>\n    </flexy-form-field>\n  ",
                    styles: ["\n      .addon {\n        display: inline-block;\n      }\n      flexy-control-number {\n        display: inline-block;\n      }\n    "]
                }] }
    ];
    FlexyFormNumberComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        prefix: [{ type: Input }],
        suffix: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        step: [{ type: Input }],
        readonly: [{ type: Input }],
        decimal: [{ type: Input }],
        refPath: [{ type: Input }]
    };
    return FlexyFormNumberComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormNumberComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.default;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.label;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.description;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.min;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.max;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.step;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.decimal;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.refPath;
    /** @type {?} */
    FlexyFormNumberComponent.prototype.refControl;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormPasswordComponent = /** @class */ (function () {
    function FlexyFormPasswordComponent() {
    }
    FlexyFormPasswordComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-password',
                    template: "\n    <flexy-form-field [control]=\"layoutSchema.formControl\" [label]=\"label\" [description]=\"description\" [ngClass]=\"{ readonly: readonly }\">\n      <flexy-control-password\n        [minLength]=\"minLength\"\n        [maxLength]=\"maxLength\"\n        [readonly]=\"readonly\"\n        [default]=\"default\"\n        [control]=\"layoutSchema.formControl\"\n      >\n      </flexy-control-password>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormPasswordComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        minLength: [{ type: Input }],
        maxLength: [{ type: Input }],
        readonly: [{ type: Input }],
        inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
    };
    return FlexyFormPasswordComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.default;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.label;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.description;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.minLength;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.maxLength;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormPasswordComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormPercentComponent = /** @class */ (function () {
    function FlexyFormPercentComponent() {
    }
    FlexyFormPercentComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-percent',
                    template: "\n    <flexy-form-field [control]=\"layoutSchema.formControl\" [label]=\"label\" [description]=\"description\" [ngClass]=\"{ readonly: readonly }\">\n      <div class=\"addon\" *ngIf=\"prefix\">{{ prefix }}</div>\n      <flexy-control-percent\n        [control]=\"layoutSchema.formControl\"\n        [min]=\"min\"\n        [max]=\"max\"\n        [step]=\"step\"\n        [default]=\"default\"\n        [readonly]=\"readonly\"\n      >\n      </flexy-control-percent>\n      <div class=\"addon\" *ngIf=\"suffix\">{{ suffix }}</div>\n    </flexy-form-field>\n  ",
                    styles: ["\n      .addon {\n        display: inline-block;\n      }\n      flexy-control-percent {\n        display: inline-block;\n      }\n    "]
                }] }
    ];
    FlexyFormPercentComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        prefix: [{ type: Input }],
        suffix: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        step: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormPercentComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormPercentComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.default;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.label;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.description;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.min;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.max;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.step;
    /** @type {?} */
    FlexyFormPercentComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormRadioListComponent = /** @class */ (function () {
    function FlexyFormRadioListComponent() {
    }
    FlexyFormRadioListComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-radio-list',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-radio-list\n        [name]=\"layoutSchema.formName\"\n        [options]=\"options\"\n        [readonly]=\"readonly\"\n        [default]=\"default\"\n        [control]=\"layoutSchema.formControl\"\n      >\n      </flexy-control-radio-list>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormRadioListComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        options: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormRadioListComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.default;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.options;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.label;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.description;
    /** @type {?} */
    FlexyFormRadioListComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormSelectComponent = /** @class */ (function () {
    function FlexyFormSelectComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormSelectComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.getOptionText(this.default) + ')';
            }
        }
    };
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    FlexyFormSelectComponent.prototype.getOptionText = /**
     * @private
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value) {
            /** @type {?} */
            var opt = this.options.find((/**
             * @param {?} item
             * @return {?}
             */
            function (item) { return '' + item.value === '' + value; }));
            return opt ? opt.text : '';
        }
        else if (value === null || value === '' || value === void 0) {
            return null;
        }
    };
    FlexyFormSelectComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-select',
                    template: "\n    <flexy-form-field [control]=\"layoutSchema.formControl\" [label]=\"label\" [description]=\"description\" [ngClass]=\"{ readonly: readonly }\">\n      <div class=\"input-group\">\n        <div class=\"input-group-addon\" *ngIf=\"prefix\">{{ prefix }}</div>\n        <flexy-control-select\n          [control]=\"layoutSchema.formControl\"\n          [default]=\"default\"\n          [readonly]=\"readonly\"\n          [options]=\"options\"\n        ></flexy-control-select>\n        <div class=\"input-group-addon\" *ngIf=\"suffix\">{{ suffix }}</div>\n      </div>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormSelectComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        options: [{ type: Input }],
        prefix: [{ type: Input }],
        suffix: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormSelectComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormSelectComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.default;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.label;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.description;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.options;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.prefix;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.suffix;
    /** @type {?} */
    FlexyFormSelectComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormSelect2Component = /** @class */ (function () {
    function FlexyFormSelect2Component(httpClient, logger, cdr, appRef) {
        this.httpClient = httpClient;
        this.logger = logger;
        this.cdr = cdr;
        this.appRef = appRef;
    }
    /**
     * @return {?}
     */
    FlexyFormSelect2Component.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.default + ')';
            }
        }
        if (this.optionsUrl) {
            this.loading = true;
            this.httpClient
                .get(this.optionsUrl)
                .pipe(map((/**
             * @param {?} data
             * @return {?}
             */
            function (data) {
                /** @type {?} */
                var options = [];
                if (data && Array.isArray(data)) {
                    data.forEach((/**
                     * @param {?} item
                     * @return {?}
                     */
                    function (item) {
                        options.push({
                            text: '' + _this.mapper(item, 'text'),
                            value: _this.mapper(item, 'value'),
                            prefixHtml: _this.optionsMapper.prefixHtml ? '' + _this.mapper(item, 'prefixHtml') : void 0
                        });
                    }));
                }
                return options;
            })))
                .subscribe((/**
             * @param {?} options
             * @return {?}
             */
            function (options) {
                _this.loading = false;
                _this.options = options;
                _this.cdr.detectChanges();
                // this.appRef.tick();
            }));
        }
    };
    /**
     * @private
     * @param {?} item
     * @param {?} key
     * @return {?}
     */
    FlexyFormSelect2Component.prototype.mapper = /**
     * @private
     * @param {?} item
     * @param {?} key
     * @return {?}
     */
    function (item, key) {
        /** @type {?} */
        var mapper = this.optionsMapper[key];
        if (!mapper) {
            this.logger.warn('Wrong mapper configuration for key', key, this.optionsMapper);
            return;
        }
        // check if template?
        if (mapper.includes('<%')) {
            /** @type {?} */
            var compiled = template(mapper);
            return compiled(item);
        }
        else {
            return get(item, mapper);
        }
    };
    FlexyFormSelect2Component.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-select2',
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [labelIcon]=\"labelIcon\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <div class=\"input-group\">\n        <div class=\"input-group-addon\" *ngIf=\"prefix\">{{ prefix }}</div>\n        <flexy-control-select2\n          [control]=\"layoutSchema.formControl\"\n          [default]=\"default\"\n          [readonly]=\"readonly\"\n          [options]=\"options\"\n          [placeholder]=\"placeholder\"\n          [multiple]=\"multiple\"\n          [addItem]=\"addItem\"\n          [hideSelected]=\"hideSelected\"\n          [enableSearchByValue]=\"enableSearchByValue\"\n          [loading]=\"loading\"\n        ></flexy-control-select2>\n        <div class=\"input-group-addon\" *ngIf=\"suffix\">{{ suffix }}</div>\n      </div>\n    </flexy-form-field>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyFormSelect2Component.ctorParameters = function () { return [
        { type: HttpClient },
        { type: FlexyLoggerService },
        { type: ChangeDetectorRef },
        { type: ApplicationRef }
    ]; };
    FlexyFormSelect2Component.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        placeholder: [{ type: Input }],
        labelIcon: [{ type: Input }],
        enableSearchByValue: [{ type: Input }],
        options: [{ type: Input }],
        optionsUrl: [{ type: Input }],
        optionsMapper: [{ type: Input }],
        prefix: [{ type: Input }],
        suffix: [{ type: Input }],
        readonly: [{ type: Input }],
        multiple: [{ type: Input }],
        addItem: [{ type: Input }],
        hideSelected: [{ type: Input }]
    };
    return FlexyFormSelect2Component;
}());
if (false) {
    /** @type {?} */
    FlexyFormSelect2Component.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.default;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.label;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.description;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.placeholder;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.labelIcon;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.enableSearchByValue;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.options;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.optionsUrl;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.optionsMapper;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.prefix;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.suffix;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.readonly;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.multiple;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.addItem;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.hideSelected;
    /** @type {?} */
    FlexyFormSelect2Component.prototype.loading;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.httpClient;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.logger;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.cdr;
    /**
     * @type {?}
     * @private
     */
    FlexyFormSelect2Component.prototype.appRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var SCHEMA_COMPONENT_ID_KEY = 'componentId';
var FlexyFormTabsComponent = /** @class */ (function () {
    function FlexyFormTabsComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormTabsComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.activeTab = this.layoutSchema.children[0] ? this.layoutSchema.children[0][SCHEMA_COMPONENT_ID_KEY] : null;
    };
    /**
     * @param {?} tab
     * @return {?}
     */
    FlexyFormTabsComponent.prototype.setActiveTab = /**
     * @param {?} tab
     * @return {?}
     */
    function (tab) {
        this.activeTab = tab.id;
    };
    FlexyFormTabsComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-tabs',
                    template: "\n    <ng-container *ngIf=\"layoutSchema && layoutSchema.children\">\n      <tabset>\n        <tab\n          *ngFor=\"let schema of layoutSchema.children\"\n          [heading]=\"schema.componentInputs?.legend\"\n          [id]=\"schema.componentId\"\n          [active]=\"activeTab === schema.componentId\"\n          (selectTab)=\"setActiveTab($event)\"\n        ></tab>\n      </tabset>\n      <ng-container *ngFor=\"let schema of layoutSchema.children\">\n        <flexy-layout *ngIf=\"activeTab === schema.componentId\" [schema]=\"schema.children\"></flexy-layout>\n      </ng-container>\n    </ng-container>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyFormTabsComponent.ctorParameters = function () { return []; };
    FlexyFormTabsComponent.propDecorators = {
        layoutSchema: [{ type: Input }]
    };
    return FlexyFormTabsComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormTabsComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTabsComponent.prototype.activeTab;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormTagsComponent = /** @class */ (function () {
    function FlexyFormTagsComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormTagsComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this.readonly) {
            if (!this.description && this.default) {
                this.description = '(default: ' + this.default + ')';
            }
        }
    };
    FlexyFormTagsComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-tags',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-tags\n        [control]=\"layoutSchema.formControl\"\n        [default]=\"default\"\n        [readonly]=\"readonly\"\n        [options]=\"options\"\n        [placeholder]=\"placeholder\"\n      ></flexy-control-tags>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormTagsComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        options: [{ type: Input }],
        placeholder: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormTagsComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormTagsComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.default;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.label;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.description;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.options;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyFormTagsComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormTextComponent = /** @class */ (function () {
    function FlexyFormTextComponent() {
    }
    FlexyFormTextComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-text',
                    template: "\n    <flexy-form-field [control]=\"layoutSchema.formControl\" [label]=\"label\" [description]=\"description\" [ngClass]=\"{ readonly: readonly }\">\n      <flexy-control-text\n        [minLength]=\"minLength\"\n        [maxLength]=\"maxLength\"\n        [readonly]=\"readonly\"\n        [default]=\"default\"\n        [control]=\"layoutSchema.formControl\"\n      >\n      </flexy-control-text>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormTextComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        minLength: [{ type: Input }],
        maxLength: [{ type: Input }],
        readonly: [{ type: Input }],
        inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
    };
    return FlexyFormTextComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormTextComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTextComponent.prototype.default;
    /** @type {?} */
    FlexyFormTextComponent.prototype.label;
    /** @type {?} */
    FlexyFormTextComponent.prototype.description;
    /** @type {?} */
    FlexyFormTextComponent.prototype.minLength;
    /** @type {?} */
    FlexyFormTextComponent.prototype.maxLength;
    /** @type {?} */
    FlexyFormTextComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormTextComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormTextareaComponent = /** @class */ (function () {
    function FlexyFormTextareaComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormTextareaComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.inputRef && this.inputRef.nativeElement) {
            this.inputRef.nativeElement.addEventListener('change', (/**
             * @param {?} e
             * @return {?}
             */
            function (e) {
                _this.layoutSchema.formControl.setValue(_this.layoutSchema.formControl.value.trim());
            }), false);
        }
    };
    FlexyFormTextareaComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-textarea',
                    template: "\n    <flexy-form-field [control]=\"layoutSchema.formControl\" [label]=\"label\" [description]=\"description\" [ngClass]=\"{ readonly: readonly }\">\n      <textarea\n        #inputRef\n        *ngIf=\"!readonly\"\n        flexyAutosize\n        class=\"form-control\"\n        [formControl]=\"layoutSchema.formControl\"\n        [attr.placeholder]=\"default\"\n      ></textarea>\n      <flexy-control-readonly *ngIf=\"readonly\" [value]=\"layoutSchema.formControl.value\" [default]=\"default\"></flexy-control-readonly>\n    </flexy-form-field>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyFormTextareaComponent.ctorParameters = function () { return []; };
    FlexyFormTextareaComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        default: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        readonly: [{ type: Input }],
        inputRef: [{ type: ViewChild, args: ['inputRef', { static: false },] }]
    };
    return FlexyFormTextareaComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.default;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.label;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.description;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.readonly;
    /** @type {?} */
    FlexyFormTextareaComponent.prototype.inputRef;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyTreeService = /** @class */ (function () {
    function FlexyTreeService() {
        this.subject = new BehaviorSubject(null);
    }
    /**
     * @param {?} node
     * @return {?}
     */
    FlexyTreeService.prototype.setSelectedId = /**
     * @param {?} node
     * @return {?}
     */
    function (node) {
        this.subject.next(node);
    };
    /**
     * @return {?}
     */
    FlexyTreeService.prototype.onChangeSelected = /**
     * @return {?}
     */
    function () {
        return this.subject.asObservable();
    };
    FlexyTreeService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    FlexyTreeService.ctorParameters = function () { return []; };
    return FlexyTreeService;
}());
if (false) {
    /**
     * @type {?}
     * @private
     */
    FlexyTreeService.prototype.subject;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyTreeComponent = /** @class */ (function () {
    function FlexyTreeComponent(treeService) {
        this.treeService = treeService;
        this.tree = [];
        this.selected = new EventEmitter();
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    FlexyTreeComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes.selectedId) {
            this.treeService.setSelectedId(this.findSelectedInChildren(this.tree, changes.selectedId.currentValue));
        }
        if (changes.tree) {
            this.nodes = this.prepareItems(changes.tree.currentValue);
        }
    };
    /**
     * @return {?}
     */
    FlexyTreeComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.selectedSubscription = this.treeService.onChangeSelected().subscribe((/**
         * @param {?} lastSelected
         * @return {?}
         */
        function (lastSelected) {
            if ((lastSelected && lastSelected.id !== _this.selectedId) || (!lastSelected && _this.selectedId)) {
                _this.selected.emit(lastSelected);
            }
            else if (!lastSelected) {
                _this.selected.emit(null);
            }
        }));
    };
    /**
     * @return {?}
     */
    FlexyTreeComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this.selectedSubscription) {
            this.selectedSubscription.unsubscribe();
        }
    };
    /**
     * @private
     * @param {?} nodes
     * @return {?}
     */
    FlexyTreeComponent.prototype.prepareItems = /**
     * @private
     * @param {?} nodes
     * @return {?}
     */
    function (nodes) {
        var _this = this;
        if (nodes && nodes.length) {
            return nodes.map((/**
             * @param {?} node
             * @return {?}
             */
            function (node) {
                return {
                    model: node,
                    isExpanded: !!_this.findSelectedInChildren(node.children, _this.selectedId),
                    isSelected: _this.selectedId === node.id,
                    isReadonly: _this.readonlyId === node.id,
                    children: node.children ? _this.prepareItems(node.children) : null
                };
            }));
        }
        return null;
    };
    /**
     * @private
     * @param {?} tree
     * @param {?} selectedId
     * @return {?}
     */
    FlexyTreeComponent.prototype.findSelectedInChildren = /**
     * @private
     * @param {?} tree
     * @param {?} selectedId
     * @return {?}
     */
    function (tree, selectedId) {
        var _this = this;
        /** @type {?} */
        var founded = null;
        if (tree) {
            tree.forEach((/**
             * @param {?} item
             * @return {?}
             */
            function (item) {
                if (item.id === selectedId) {
                    founded = item;
                    return;
                }
                else if (item.children) {
                    /** @type {?} */
                    var f = _this.findSelectedInChildren(item.children, selectedId);
                    if (f) {
                        founded = f;
                        return;
                    }
                }
            }));
        }
        return founded;
    };
    FlexyTreeComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-tree',
                    template: "\n    <flexy-tree-node *ngIf=\"nodes\" [nodes]=\"nodes\" [templateRef]=\"templateRef\"></flexy-tree-node>\n  ",
                    providers: [FlexyTreeService]
                }] }
    ];
    /** @nocollapse */
    FlexyTreeComponent.ctorParameters = function () { return [
        { type: FlexyTreeService }
    ]; };
    FlexyTreeComponent.propDecorators = {
        tree: [{ type: Input }],
        selectedId: [{ type: Input }],
        readonlyId: [{ type: Input }],
        templateRef: [{ type: Input }],
        selected: [{ type: Output }]
    };
    return FlexyTreeComponent;
}());
if (false) {
    /** @type {?} */
    FlexyTreeComponent.prototype.tree;
    /** @type {?} */
    FlexyTreeComponent.prototype.selectedId;
    /** @type {?} */
    FlexyTreeComponent.prototype.readonlyId;
    /** @type {?} */
    FlexyTreeComponent.prototype.templateRef;
    /** @type {?} */
    FlexyTreeComponent.prototype.selected;
    /** @type {?} */
    FlexyTreeComponent.prototype.nodes;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeComponent.prototype.selectedSubscription;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeComponent.prototype.treeService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyTreeNodeComponent = /** @class */ (function () {
    function FlexyTreeNodeComponent(treeService) {
        this.treeService = treeService;
    }
    /**
     * @return {?}
     */
    FlexyTreeNodeComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._subscription = this.treeService.onChangeSelected().subscribe((/**
         * @param {?} lastSelected
         * @return {?}
         */
        function (lastSelected) {
            if (_this.nodes) {
                _this.nodes.forEach((/**
                 * @param {?} item
                 * @return {?}
                 */
                function (item) {
                    item.isSelected = lastSelected && item.model.id === lastSelected.id;
                }));
            }
        }));
    };
    /**
     * @return {?}
     */
    FlexyTreeNodeComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this._subscription) {
            this._subscription.unsubscribe();
        }
    };
    /**
     * @param {?} leaf
     * @return {?}
     */
    FlexyTreeNodeComponent.prototype.toggleExpand = /**
     * @param {?} leaf
     * @return {?}
     */
    function (leaf) {
        leaf.isExpanded = !leaf.isExpanded;
    };
    /**
     * @param {?} node
     * @return {?}
     */
    FlexyTreeNodeComponent.prototype.toggleSelected = /**
     * @param {?} node
     * @return {?}
     */
    function (node) {
        node.isSelected = !node.isSelected;
        this.treeService.setSelectedId(node.isSelected ? node.model : null);
    };
    FlexyTreeNodeComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-tree-node',
                    template: "\n    <ul class=\"tree\" *ngIf=\"nodes\">\n      <li class=\"node\" *ngFor=\"let item of nodes\" [ngClass]=\"{ active: item.isSelected && !item.isReadonly, readonly: item.isReadonly }\">\n        <a\n          (click)=\"toggleExpand(item)\"\n          *ngIf=\"item.model && item.model.hasChildren()\"\n          role=\"button\"\n          class=\"fa expand-group t2e-btn-expand\"\n          [ngClass]=\"{\n            'fa-angle-right': item.model && !item.isExpanded,\n            'fa-angle-down': item.model && item.isExpanded\n          }\"\n        ></a>\n        <div class=\"inside t2e-btn-toggle-check\" (click)=\"toggleSelected(item)\">\n          <span role=\"button\" *ngIf=\"templateRef\">\n            <ng-container *ngTemplateOutlet=\"templateRef; context: { $implicit: item.model, isSelected: item.isSelected }\"> </ng-container>\n          </span>\n          <span *ngIf=\"!templateRef\"> [{{ item.model.id }}] </span>\n        </div>\n        <div *ngIf=\"item.isExpanded && item.children\">\n          <flexy-tree-node [templateRef]=\"templateRef\" [nodes]=\"item.children\"></flexy-tree-node>\n        </div>\n      </li>\n    </ul>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyTreeNodeComponent.ctorParameters = function () { return [
        { type: FlexyTreeService }
    ]; };
    FlexyTreeNodeComponent.propDecorators = {
        nodes: [{ type: Input }],
        templateRef: [{ type: Input }]
    };
    return FlexyTreeNodeComponent;
}());
if (false) {
    /** @type {?} */
    FlexyTreeNodeComponent.prototype.nodes;
    /** @type {?} */
    FlexyTreeNodeComponent.prototype.templateRef;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeNodeComponent.prototype._subscription;
    /**
     * @type {?}
     * @private
     */
    FlexyTreeNodeComponent.prototype.treeService;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormTreeSelectComponent = /** @class */ (function () {
    function FlexyFormTreeSelectComponent() {
    }
    /**
     * @return {?}
     */
    FlexyFormTreeSelectComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    FlexyFormTreeSelectComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-tree-select',
                    template: "\n    <flexy-form-field\n      [control]=\"layoutSchema.formControl\"\n      [name]=\"layoutSchema.formName\"\n      [label]=\"label\"\n      [labelIcon]=\"labelIcon\"\n      [description]=\"description\"\n      [ngClass]=\"{ readonly: readonly }\"\n    >\n      <flexy-control-tree-select\n        [control]=\"layoutSchema.formControl\"\n        [dataTree]=\"dataTree\"\n        [default]=\"default\"\n        [readonlyId]=\"readonlyId\"\n        [labelKey]=\"labelKey\"\n        [readonly]=\"readonly\"\n      >\n      </flexy-control-tree-select>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormTreeSelectComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        dataTree: [{ type: Input }],
        default: [{ type: Input }],
        readonlyId: [{ type: Input }],
        labelKey: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        labelIcon: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormTreeSelectComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.dataTree;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.default;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.readonlyId;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.labelKey;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.label;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.description;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.labelIcon;
    /** @type {?} */
    FlexyFormTreeSelectComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlCheckboxComponent = /** @class */ (function () {
    function FlexyControlCheckboxComponent() {
        this.isValueDefined = false;
        this.isDefaultDefined = false;
    }
    /**
     * @return {?}
     */
    FlexyControlCheckboxComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.isValueDefined = this.control && this.control.value !== null && this.control.value !== void 0;
        this.isDefaultDefined = this.default !== void 0;
        if (!this.readonly && !this.isValueDefined && this.isDefaultDefined) {
            this.control.setValue(this.default);
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyControlCheckboxComponent.prototype.onFocus = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        var _this = this;
        event.target.onkeydown = (/**
         * @param {?} keyEvent
         * @return {?}
         */
        function (keyEvent) {
            if (keyEvent.code === 'Space') {
                keyEvent.preventDefault();
                keyEvent.stopPropagation();
                _this.toogle();
            }
        });
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyControlCheckboxComponent.prototype.onBlur = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        event.target.onkeydown = null;
    };
    /**
     * @return {?}
     */
    FlexyControlCheckboxComponent.prototype.toogle = /**
     * @return {?}
     */
    function () {
        this.control.markAsDirty();
        this.control.setValue(!this.control.value);
    };
    FlexyControlCheckboxComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-control-checkbox',
                    template: "\n    <a\n      *ngIf=\"!readonly && onlyLabel\"\n      (focus)=\"onFocus($event)\"\n      (blur)=\"onBlur($event)\"\n      (click)=\"toogle()\"\n      [ngClass]=\"{ active: control.value }\"\n    >\n      <i *ngIf=\"faIcon\" [class]=\"faIcon\"></i> {{ onlyLabel }}\n    </a>\n    <a\n      role=\"checkbox\"\n      tabindex=\"0\"\n      *ngIf=\"!readonly && !onlyLabel\"\n      class=\"fa\"\n      (focus)=\"onFocus($event)\"\n      (blur)=\"onBlur($event)\"\n      (click)=\"toogle(); $event.stopPropagation()\"\n      [ngClass]=\"{\n        'fa-check-square-o': control.value,\n        'fa-square-o': !control.value,\n        'ng-dirty': control.dirty,\n        'ng-invalid': !control.valid\n      }\"\n      role=\"button\"\n    ></a>\n    <span\n      *ngIf=\"readonly && isValueDefined\"\n      class=\"fa\"\n      [ngClass]=\"{\n        'fa-check': control.value === true,\n        'fa-minus': control.value === false\n      }\"\n    ></span>\n    <div *ngIf=\"isDefaultDefined && !isValueDefined\">\n      <small>\n        <span *ngIf=\"readonly\" class=\"fa\" [ngClass]=\"{ 'fa-check': default, 'fa-minus': !default }\"></span>\n        <span *ngIf=\"!readonly && !control.dirty\">default</span>\n      </small>\n    </div>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyControlCheckboxComponent.ctorParameters = function () { return []; };
    FlexyControlCheckboxComponent.propDecorators = {
        control: [{ type: Input }],
        readonly: [{ type: Input }],
        default: [{ type: Input }],
        onlyLabel: [{ type: Input }],
        faIcon: [{ type: Input }]
    };
    return FlexyControlCheckboxComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.control;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.default;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.onlyLabel;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.faIcon;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.isValueDefined;
    /** @type {?} */
    FlexyControlCheckboxComponent.prototype.isDefaultDefined;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlCheckboxListComponent = /** @class */ (function () {
    function FlexyControlCheckboxListComponent(formBuilder) {
        this.formBuilder = formBuilder;
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlCheckboxListComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.initCheckboxesGroup();
        this.setDefaultsLabels();
        this.setValueLabels();
    };
    /**
     * @return {?}
     */
    FlexyControlCheckboxListComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this.controlChangesSubscription) {
            this.controlChangesSubscription.unsubscribe();
        }
        if (this.localValuesChangesSubscription) {
            this.localValuesChangesSubscription.unsubscribe();
        }
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlCheckboxListComponent.prototype.initCheckboxesGroup = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var formConfig = {};
        if (this.options) {
            /** @type {?} */
            var values_1 = this.control.value || [];
            this.options.forEach((/**
             * @param {?} option
             * @return {?}
             */
            function (option) {
                formConfig[option.value] = [values_1.includes(option.value), []];
            }));
        }
        this.localGroupControl = this.formBuilder.group(formConfig);
        this.controlChangesSubscription = this.control.valueChanges.subscribe((/**
         * @param {?} values
         * @return {?}
         */
        function (values) {
            if (_this.localValuesChangesSubscription) {
                _this.localValuesChangesSubscription.unsubscribe();
            }
            _this.options.forEach((/**
             * @param {?} option
             * @return {?}
             */
            function (option) {
                _this.localGroupControl.get(option.value).setValue(values && values.includes(option.value) ? true : null);
            }));
            _this.subscribeLocalChanges();
        }));
        this.subscribeLocalChanges();
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlCheckboxListComponent.prototype.subscribeLocalChanges = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        this.localValuesChangesSubscription = this.localGroupControl.valueChanges.subscribe((/**
         * @param {?} values
         * @return {?}
         */
        function (values) {
            _this.updateControl();
        }));
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlCheckboxListComponent.prototype.setDefaultsLabels = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.default) {
            this.defaultsLabels = this.options
                .filter((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return _this.default.includes(option.value); }))
                .map((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return option.text; }))
                .join(', ');
        }
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlCheckboxListComponent.prototype.setValueLabels = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.options) {
            this.valuesLabels = this.options
                .filter((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return _this.control.value && _this.control.value.includes(option.value); }))
                .map((/**
             * @param {?} option
             * @return {?}
             */
            function (option) { return option.text; }))
                .join(', ');
        }
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlCheckboxListComponent.prototype.updateControl = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var val = this.options.filter((/**
         * @param {?} option
         * @return {?}
         */
        function (option) { return !!_this.localGroupControl.get(option.value).value; })).map((/**
         * @param {?} option
         * @return {?}
         */
        function (option) { return option.value; }));
        this.control.setValue(val.length ? val : null);
        this.control.markAsDirty();
        this.setValueLabels();
        this.changed.emit(this.control.value);
    };
    FlexyControlCheckboxListComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <div *ngIf=\"!readonly\" class=\"input-group form-control\">\n      <div *ngFor=\"let option of options; let i = index\" class=\"checkboxItem\">\n        <input\n          type=\"checkbox\"\n          [id]=\"name + '-' + option.value\"\n          [value]=\"option.value\"\n          [formControl]=\"localGroupControl.get(option.value)\"\n        /><label for=\"{{ name + '-' + option.value }}\">{{ option.text }}</label>\n      </div>\n    </div>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"valuesLabels\" [default]=\"defaultsLabels\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-checkbox-list'
                }] }
    ];
    /** @nocollapse */
    FlexyControlCheckboxListComponent.ctorParameters = function () { return [
        { type: FormBuilder }
    ]; };
    FlexyControlCheckboxListComponent.propDecorators = {
        control: [{ type: Input }],
        name: [{ type: Input }],
        default: [{ type: Input }],
        options: [{ type: Input }],
        readonly: [{ type: Input }],
        changed: [{ type: Output }]
    };
    return FlexyControlCheckboxListComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.control;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.name;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.default;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.options;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.changed;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.localGroupControl;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.valuesLabels;
    /** @type {?} */
    FlexyControlCheckboxListComponent.prototype.defaultsLabels;
    /**
     * @type {?}
     * @private
     */
    FlexyControlCheckboxListComponent.prototype.controlChangesSubscription;
    /**
     * @type {?}
     * @private
     */
    FlexyControlCheckboxListComponent.prototype.localValuesChangesSubscription;
    /**
     * @type {?}
     * @private
     */
    FlexyControlCheckboxListComponent.prototype.formBuilder;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlChipsComponent = /** @class */ (function () {
    function FlexyControlChipsComponent() {
        this.autocompleteItems = [];
        this.disableDragging = false;
        this.animations = false;
        this.onlyFromAutocomplete = false;
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlChipsComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.animationDuration = !this.animations ? { enter: '0ms', leave: '0ms' } : { enter: '250ms', leave: '150ms' };
        this.setFormControl();
    };
    /**
     * @return {?}
     */
    FlexyControlChipsComponent.prototype.onChange = /**
     * @return {?}
     */
    function () {
        this.control.setValue(this.chipsFormControl.value.map((/**
         * @param {?} val
         * @return {?}
         */
        function (val) { return (val === 'null' ? null : val); })));
        this.control.markAsDirty();
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlChipsComponent.prototype.setFormControl = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var control = cloneDeep(this.control);
        control.setValue(control.value ? control.value.map((/**
         * @param {?} val
         * @return {?}
         */
        function (val) { return String(val); })) : []);
        this.chipsFormControl = control;
        this.autocompleteItems = this.autocompleteItems && this.autocompleteItems.length ? this.autocompleteItems : control.value;
    };
    FlexyControlChipsComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-control-chips',
                    template: "\n    <tag-input\n      *ngIf=\"!readonly\"\n      [formControl]=\"chipsFormControl\"\n      [editable]=\"true\"\n      [dragZone]=\"!disableDragging ? 'dragzone' : null\"\n      [modelAsStrings]=\"true\"\n      [animationDuration]=\"animationDuration\"\n      [placeholder]=\"placeholder\"\n      [secondaryPlaceholder]=\"placeholder\"\n      [theme]=\"'ngx-chips-custom-theme'\"\n      [addOnBlur]=\"true\"\n      [onlyFromAutocomplete]=\"onlyFromAutocomplete\"\n      (onTagEdited)=\"onChange()\"\n      (onRemove)=\"onChange()\"\n      (onAdd)=\"onChange()\"\n    >\n      <tag-input-dropdown [autocompleteItems]=\"autocompleteItems\" [showDropdownIfEmpty]=\"true\" [appendToBody]=\"false\"></tag-input-dropdown>\n    </tag-input>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"chipsFormControl?.value.join(', ')\" [default]=\"default\"> </flexy-control-readonly>\n  "
                }] }
    ];
    FlexyControlChipsComponent.propDecorators = {
        control: [{ type: Input }],
        autocompleteItems: [{ type: Input }],
        default: [{ type: Input }],
        readonly: [{ type: Input }],
        placeholder: [{ type: Input }],
        disableDragging: [{ type: Input }],
        animations: [{ type: Input }],
        onlyFromAutocomplete: [{ type: Input }],
        changed: [{ type: Output }]
    };
    return FlexyControlChipsComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlChipsComponent.prototype.control;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.autocompleteItems;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.default;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.disableDragging;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.animations;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.onlyFromAutocomplete;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.changed;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.animationDuration;
    /** @type {?} */
    FlexyControlChipsComponent.prototype.chipsFormControl;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlColorpickerComponent = /** @class */ (function () {
    function FlexyControlColorpickerComponent() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    FlexyControlColorpickerComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <div class=\"form-control color-picker-container\">\n      <div class=\"form-control color-picker-input\">\n        <input\n          *ngIf=\"!readonly\"\n          class=\"notreadonly\"\n          type=\"color\"\n          list=\"presetColors\"\n          [value]=\"control.value ? control.value : default\"\n          [formControl]=\"control\"\n          (focus)=\"focused.emit($event)\"\n          (click)=\"clicked.emit($event)\"\n          (changed)=\"changed.emit($event)\"\n        />\n        <datalist id=\"presetColors\" *ngIf=\"!readonly\">\n          <option *ngFor=\"let color of list\">{{ color }}</option>\n        </datalist>\n        <input type=\"color\" *ngIf=\"readonly\" [value]=\"control.value ? control.value : default\" disabled=\"true\" />\n      </div>\n    </div>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"control.value\" [default]=\"default\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-colorpicker'
                }] }
    ];
    FlexyControlColorpickerComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        list: [{ type: Input }],
        readonly: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlColorpickerComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.control;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.default;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.list;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.focused;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlColorpickerComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var DATEPICKER_THEMES = ['theme-default', 'theme-green', 'theme-blue', 'theme-dark-blue', 'theme-red', 'theme-orange'];
/** @type {?} */
var BS_DATEPICKER_MODES = {
    month: 'month',
    quarter: 'month',
    year: 'year',
    day: 'day'
};
var FlexyControlDatepickerComponent = /** @class */ (function () {
    function FlexyControlDatepickerComponent() {
        this.focus = new EventEmitter();
        this.click = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlDatepickerComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.configDatepicker();
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlDatepickerComponent.prototype.configDatepicker = /**
     * @private
     * @return {?}
     */
    function () {
        this.minDate = this.min ? new Date(this.min) : null;
        this.maxDate = this.max ? new Date(this.max) : null;
        this.bsConfig = {
            minMode: this.minMode && BS_DATEPICKER_MODES[this.minMode] ? BS_DATEPICKER_MODES[this.minMode] : 'day',
            dateInputFormat: this.format ? this.format.toUpperCase() : 'YYYY-MM-DD',
            containerClass: [DATEPICKER_THEMES.includes(this.theme) ? this.theme : 'theme-default', 'mode-' + this.minMode]
        };
    };
    FlexyControlDatepickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-control-datepicker',
                    template: "\n    <div class=\"input-group\" *ngIf=\"!readonly\">\n      <input\n        #dp=\"bsDatepicker\"\n        class=\"form-control\"\n        type=\"text\"\n        [isDisabled]=\"isDisabled\"\n        [formControl]=\"control\"\n        [attr.placeholder]=\"placeholder\"\n        [bsConfig]=\"bsConfig\"\n        [minDate]=\"minDate\"\n        [maxDate]=\"maxDate\"\n        (focus)=\"focus.emit($event)\"\n        (click)=\"click.emit($event)\"\n        (bsValueChange)=\"changed.emit($event)\"\n        bsDatepicker\n      />\n      <span class=\"input-group-btn\">\n        <button *ngIf=\"!hideButton\" class=\"btn btn-primary\" (click)=\"dp.toggle(); $event.stopPropagation()\">\n          <span class=\"flexy-icon-calendar\"></span>\n        </button>\n      </span>\n    </div>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"control.value | date: format\" [default]=\"default\"> </flexy-control-readonly>\n  "
                }] }
    ];
    FlexyControlDatepickerComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        placeholder: [{ type: Input }],
        theme: [{ type: Input }],
        format: [{ type: Input }],
        readonly: [{ type: Input }],
        isDisabled: [{ type: Input }],
        hideButton: [{ type: Input }],
        minMode: [{ type: Input }],
        focus: [{ type: Output }],
        click: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlDatepickerComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.bsConfig;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.bsValue;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.minDate;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.maxDate;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.defaultFormatted;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.control;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.default;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.min;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.max;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.format;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.hideButton;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.minMode;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.focus;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.click;
    /** @type {?} */
    FlexyControlDatepickerComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var moment = momentNs;
/** @type {?} */
var DATEPICKER_THEMES$1 = ['theme-default', 'theme-green', 'theme-blue', 'theme-dark-blue', 'theme-red', 'theme-orange'];
var FlexyControlDaterangepickerComponent = /** @class */ (function () {
    function FlexyControlDaterangepickerComponent() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlDaterangepickerComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.configDatepicker();
        this.setReadonlyValue();
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyControlDaterangepickerComponent.prototype.onChange = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.setReadonlyValue();
        this.changed.emit(event);
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlDaterangepickerComponent.prototype.configDatepicker = /**
     * @private
     * @return {?}
     */
    function () {
        if (this.default && this.default.start && this.default.end) {
            this.defaultRange = this.default.start + ' - ' + this.default.end;
        }
        this.minDate = this.min ? new Date(this.min) : null;
        this.maxDate = this.max ? new Date(this.max) : null;
        this.bsConfig = {
            rangeInputFormat: this.format ? this.format.toUpperCase() : 'YYYY-MM-DD',
            containerClass: [DATEPICKER_THEMES$1.includes(this.theme) ? this.theme : 'theme-default', this.customClass]
        };
    };
    /**
     * @private
     * @return {?}
     */
    FlexyControlDaterangepickerComponent.prototype.setReadonlyValue = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.control.value && this.format) {
            this.readonlyValue = this.control.value.map((/**
             * @param {?} val
             * @return {?}
             */
            function (val) { return moment(val).format(_this.format.toUpperCase()); })).join(' - ');
        }
    };
    FlexyControlDaterangepickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-control-daterangepicker',
                    template: "\n    <div class=\"input-group\" *ngIf=\"!readonly\">\n      <input\n        #drp=\"bsDaterangepicker\"\n        class=\"form-control\"\n        type=\"text\"\n        [isDisabled]=\"isDisabled\"\n        [formControl]=\"control\"\n        [attr.placeholder]=\"placeholder\"\n        [bsConfig]=\"bsConfig\"\n        [minDate]=\"minDate\"\n        [maxDate]=\"maxDate\"\n        (focus)=\"focused.emit($event)\"\n        (click)=\"clicked.emit($event)\"\n        (bsValueChange)=\"onChange($event)\"\n        bsDaterangepicker\n      />\n      <span class=\"input-group-btn\">\n        <button class=\"btn btn-primary\" (click)=\"drp.toggle(); $event.stopPropagation()\">\n          <span class=\"flexy-icon-calendar\"></span>\n        </button>\n      </span>\n    </div>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"readonlyValue\" [default]=\"defaultRange\"></flexy-control-readonly>\n  "
                }] }
    ];
    FlexyControlDaterangepickerComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        placeholder: [{ type: Input }],
        theme: [{ type: Input }],
        format: [{ type: Input }],
        readonly: [{ type: Input }],
        isDisabled: [{ type: Input }],
        customClass: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlDaterangepickerComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.control;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.default;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.min;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.max;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.theme;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.format;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.isDisabled;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.customClass;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.focused;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.changed;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.bsConfig;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.bsRangeValue;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.minDate;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.maxDate;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.defaultRange;
    /** @type {?} */
    FlexyControlDaterangepickerComponent.prototype.readonlyValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlDraggableSelectComponent = /** @class */ (function () {
    function FlexyControlDraggableSelectComponent() {
        this.optionsMap = {};
        this.showSelect = false;
    }
    /**
     * @return {?}
     */
    FlexyControlDraggableSelectComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.options.forEach((/**
         * @param {?} option
         * @return {?}
         */
        function (option) {
            _this.optionsMap[option.value] = option.text;
        }));
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyControlDraggableSelectComponent.prototype.drop = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        moveItemInArray(this.control.value, event.previousIndex, event.currentIndex);
        this.control.markAsDirty();
    };
    /**
     * @param {?} index
     * @return {?}
     */
    FlexyControlDraggableSelectComponent.prototype.removeItem = /**
     * @param {?} index
     * @return {?}
     */
    function (index) {
        /** @type {?} */
        var value = this.control.value;
        value.splice(index, 1);
        this.control.setValue(value);
        this.control.markAsDirty();
    };
    /**
     * @return {?}
     */
    FlexyControlDraggableSelectComponent.prototype.isDraggable = /**
     * @return {?}
     */
    function () {
        return !this.readonly && this.control.value && this.control.value.length > 1;
    };
    FlexyControlDraggableSelectComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-control-draggable-select',
                    template: "\n    <div class=\"draggable-list-container\">\n      <table cdkDropList class=\"drag-container\" [cdkDropListDisabled]=\"!isDraggable()\" (cdkDropListDropped)=\"drop($event)\">\n        <tr\n          *ngFor=\"let value of control.value; let index = index\"\n          cdkDrag\n          cdkDragLockAxis=\"y\"\n          class=\"flexy-form-select-draggable-item\"\n          [ngClass]=\"{ draggable: isDraggable() }\"\n          cdkDragBoundary=\".drag-container\"\n        >\n          <td>\n            {{ optionsMap[value] }}\n            <span *ngIf=\"!optionsMap[value]\" class=\"unknown-value\">{{ 'FLEXY_FORM_UNKNOWN_VALUE' | translate: { value: value } }}</span>\n          </td>\n          <td class=\"draggable-item-remove\">\n            <button\n              *ngIf=\"!readonly\"\n              type=\"button\"\n              class=\"btn btn-outline delete-select-item e2e-btn-delete-select-item\"\n              tooltip=\"{{ 'FLEXY_FORM_ARRAY_REMOVE_ITEM' | translate }}\"\n              (click)=\"removeItem(index)\"\n            >\n              <i class=\"flexy-icon-remove\"></i>\n            </button>\n          </td>\n        </tr>\n      </table>\n    </div>\n    <button\n      *ngIf=\"!readonly && !showSelect\"\n      type=\"button\"\n      class=\"btn btn-info btn-outline btn-sm bottom-button\"\n      (click)=\"showSelect = !showSelect\"\n    >\n      <i class=\"flexy-icon-plus\"></i> {{ 'FLEXY_FORM_ARRAY_ADD' | translate }}\n    </button>\n    <flexy-control-select2\n      *ngIf=\"showSelect\"\n      [control]=\"control\"\n      [default]=\"default\"\n      [options]=\"options\"\n      [multiple]=\"true\"\n      [hideSelected]=\"hideSelected\"\n      [enableSearchByValue]=\"enableSearchByValue\"\n      (changed)=\"showSelect = false\"\n    ></flexy-control-select2>\n  "
                }] }
    ];
    FlexyControlDraggableSelectComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        options: [{ type: Input }],
        placeholder: [{ type: Input }],
        hideSelected: [{ type: Input }],
        enableSearchByValue: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyControlDraggableSelectComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.control;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.default;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.options;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.hideSelected;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.enableSearchByValue;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.optionsMap;
    /** @type {?} */
    FlexyControlDraggableSelectComponent.prototype.showSelect;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlNumberComponent = /** @class */ (function () {
    function FlexyControlNumberComponent() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    FlexyControlNumberComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <input\n      *ngIf=\"!readonly\"\n      class=\"form-control\"\n      type=\"number\"\n      [formControl]=\"control\"\n      [attr.min]=\"min\"\n      [attr.max]=\"max\"\n      [attr.step]=\"step ? step : 1\"\n      [attr.placeholder]=\"default\"\n      (focus)=\"focused.emit($event)\"\n      (click)=\"clicked.emit($event)\"\n      (change)=\"changed.emit($event)\"\n    />\n    <flexy-control-readonly\n      *ngIf=\"readonly\"\n      [value]=\"decimal ? (control.value | number: decimal) : control.value\"\n      [default]=\"default\"\n    ></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-number',
                    styles: ["\n      input {\n        width: 10em;\n      }\n    "]
                }] }
    ];
    FlexyControlNumberComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        step: [{ type: Input }],
        readonly: [{ type: Input }],
        decimal: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlNumberComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlNumberComponent.prototype.control;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.default;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.min;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.max;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.step;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.decimal;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.focused;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlNumberComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlPasswordComponent = /** @class */ (function () {
    function FlexyControlPasswordComponent() {
        this.autocomplete = '';
        this.onFocus = new EventEmitter();
        this.onClick = new EventEmitter();
        this.onChanged = new EventEmitter();
    }
    FlexyControlPasswordComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <input\n      *ngIf=\"!readonly\"\n      class=\"form-control\"\n      type=\"password\"\n      [formControl]=\"control\"\n      [attr.minLength]=\"minLength\"\n      [attr.maxLength]=\"maxLength\"\n      [attr.placeholder]=\"default\"\n      [attr.autocomplete]=\"'new-password'\"\n      (focus)=\"onFocus.emit($event)\"\n      (click)=\"onClick.emit($event)\"\n      (change)=\"onChanged.emit($event)\"\n    />\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"control.value\" [default]=\"default\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-password'
                }] }
    ];
    FlexyControlPasswordComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        minLength: [{ type: Input }],
        maxLength: [{ type: Input }],
        readonly: [{ type: Input }],
        autocomplete: [{ type: Input }],
        onFocus: [{ type: Output }],
        onClick: [{ type: Output }],
        onChanged: [{ type: Output }]
    };
    return FlexyControlPasswordComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.control;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.default;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.minLength;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.maxLength;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.autocomplete;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.onFocus;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.onClick;
    /** @type {?} */
    FlexyControlPasswordComponent.prototype.onChanged;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlPercentComponent = /** @class */ (function () {
    function FlexyControlPercentComponent() {
        this.onFocus = new EventEmitter();
        this.onClick = new EventEmitter();
        this.onChanged = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlPercentComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.percentValue = typeof this.control.value === 'number' ? this.control.value * 100 : null;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyControlPercentComponent.prototype.onChange = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.control.setValue(this.percentValue / 100);
        this.onChanged.emit(event);
    };
    FlexyControlPercentComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <input\n      *ngIf=\"!readonly\"\n      class=\"form-control\"\n      type=\"number\"\n      [(ngModel)]=\"percentValue\"\n      [attr.min]=\"min\"\n      [attr.max]=\"max\"\n      [attr.step]=\"step ? step : 1\"\n      [attr.placeholder]=\"default\"\n      (focus)=\"onFocus.emit($event)\"\n      (click)=\"onClick.emit($event)\"\n      (change)=\"onChange($event)\"\n    />\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"percentValue\" [default]=\"default\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-percent',
                    styles: ["\n      input {\n        width: 10em;\n      }\n    "]
                }] }
    ];
    FlexyControlPercentComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        min: [{ type: Input }],
        max: [{ type: Input }],
        step: [{ type: Input }],
        readonly: [{ type: Input }],
        onFocus: [{ type: Output }],
        onClick: [{ type: Output }],
        onChanged: [{ type: Output }]
    };
    return FlexyControlPercentComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlPercentComponent.prototype.control;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.default;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.min;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.max;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.step;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.onFocus;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.onClick;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.onChanged;
    /** @type {?} */
    FlexyControlPercentComponent.prototype.percentValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlRadioListComponent = /** @class */ (function () {
    function FlexyControlRadioListComponent() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    FlexyControlRadioListComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <div *ngIf=\"!readonly\" class=\"input-group form-control\">\n      <div *ngFor=\"let option of options\" class=\"radioItem\">\n        <input\n          type=\"radio\"\n          [formControl]=\"control\"\n          [id]=\"name + '-' + option.value\"\n          [name]=\"name\"\n          [value]=\"option.value\"\n          (focus)=\"focused.emit($event)\"\n          (click)=\"clicked.emit($event)\"\n          (change)=\"changed.emit($event)\"\n        /><label for=\"{{ name + '-' + option.value }}\">{{ option.text }}</label>\n      </div>\n    </div>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"control.value\" [default]=\"default\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-radio-list'
                }] }
    ];
    FlexyControlRadioListComponent.propDecorators = {
        control: [{ type: Input }],
        name: [{ type: Input }],
        default: [{ type: Input }],
        options: [{ type: Input }],
        readonly: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlRadioListComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.control;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.name;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.default;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.options;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.focused;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlRadioListComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlReadonlyComponent = /** @class */ (function () {
    function FlexyControlReadonlyComponent() {
        this.isHtml = false;
    }
    FlexyControlReadonlyComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <span>\n      <ng-container *ngIf=\"!isHtml\">{{ value }}</ng-container>\n      <span *ngIf=\"isHtml\" [innerHTML]=\"value\"></span>\n      <i *ngIf=\"value === null || value === undefined\">{{ default }}</i>\n    </span>\n  ",
                    selector: 'flexy-control-readonly'
                }] }
    ];
    FlexyControlReadonlyComponent.propDecorators = {
        value: [{ type: Input }],
        default: [{ type: Input }],
        isHtml: [{ type: Input }]
    };
    return FlexyControlReadonlyComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlReadonlyComponent.prototype.value;
    /** @type {?} */
    FlexyControlReadonlyComponent.prototype.default;
    /** @type {?} */
    FlexyControlReadonlyComponent.prototype.isHtml;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlSelectComponent = /** @class */ (function () {
    function FlexyControlSelectComponent() {
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlSelectComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var optionsData = [{ value: null, text: '--' }];
        if (this.options) {
            optionsData.push.apply(optionsData, __spread(this.options.map((/**
             * @param {?} item
             * @return {?}
             */
            function (item) {
                /** @type {?} */
                var itemData;
                if (typeof item === 'object' && item.hasOwnProperty('value')) {
                    itemData = item;
                }
                else {
                    itemData = {
                        value: item,
                        text: '' + item
                    };
                }
                return itemData;
            }))));
        }
        this.optionsData = optionsData;
    };
    /**
     * @param {?} data
     * @return {?}
     */
    FlexyControlSelectComponent.prototype.onChanged = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        this.changed.emit(data.value);
    };
    /**
     * @param {?} value
     * @return {?}
     */
    FlexyControlSelectComponent.prototype.getOptionText = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value || value === 0) {
            /** @type {?} */
            var opt = this.options.find((/**
             * @param {?} item
             * @return {?}
             */
            function (item) { return '' + item.value === '' + value; }));
            return opt ? opt.text : '';
        }
        else if (value === null || value === '' || value === void 0) {
            return null;
        }
    };
    FlexyControlSelectComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <select\n      *ngIf=\"!readonly\"\n      class=\"form-control\"\n      [formControl]=\"control\"\n      [attr.placeholder]=\"getOptionText(default)\"\n      (focus)=\"focused.emit($event)\"\n      (click)=\"clicked.emit($event)\"\n      (change)=\"onChanged($event)\"\n    >\n      <option *ngFor=\"let option of optionsData\" [ngValue]=\"option.value\">\n        {{ option.text }}\n      </option>\n    </select>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"getOptionText(control.value)\" [default]=\"getOptionText(default)\">\n    </flexy-control-readonly>\n  ",
                    selector: 'flexy-control-select'
                }] }
    ];
    FlexyControlSelectComponent.propDecorators = {
        control: [{ type: Input }],
        options: [{ type: Input }],
        default: [{ type: Input }],
        readonly: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlSelectComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlSelectComponent.prototype.control;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.options;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.default;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.focused;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.changed;
    /** @type {?} */
    FlexyControlSelectComponent.prototype.optionsData;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlSelect2Component = /** @class */ (function () {
    function FlexyControlSelect2Component() {
        this.loading = false; // no	You can set the loading state from the outside (e.g. async items loading)
        // no	You can set the loading state from the outside (e.g. async items loading)
        this.loadingText = 'Loading...'; // Loading...	no	Set custom text when for loading items
        // Loading...	no	Set custom text when for loading items
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
        this.virtualScroll = false; // Enable virtual scroll for better performance when rendering a lot of data
    }
    /**
     * @return {?}
     */
    FlexyControlSelect2Component.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    /**
     * @param {?} changes
     * @return {?}
     */
    FlexyControlSelect2Component.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if (changes.options) {
            /** @type {?} */
            var optionsData = [];
            if (this.options) {
                optionsData.push.apply(optionsData, __spread(this.options.map((/**
                 * @param {?} item
                 * @return {?}
                 */
                function (item) {
                    /** @type {?} */
                    var itemData;
                    if (typeof item === 'object' && item.hasOwnProperty('value')) {
                        itemData = item;
                    }
                    else {
                        itemData = {
                            value: item,
                            text: '' + item
                        };
                    }
                    return itemData;
                }))));
            }
            this.virtualScroll = optionsData.length > 100;
            this.optionsData = optionsData;
        }
    };
    /**
     * @param {?} data
     * @return {?}
     */
    FlexyControlSelect2Component.prototype.onChange = /**
     * @param {?} data
     * @return {?}
     */
    function (data) {
        /** @type {?} */
        var value = data ? (Array.isArray(data) ? data.map((/**
         * @param {?} el
         * @return {?}
         */
        function (el) { return el.value; })) : data.value) : null;
        this.control.setValue(value);
        this.changed.emit(value);
    };
    /**
     * @param {?} value
     * @return {?}
     */
    FlexyControlSelect2Component.prototype.getReadonlyInfo = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value && this.addItem) {
            return value;
        }
        if ((value || value === 0) && this.optionsData) {
            /** @type {?} */
            var opt = this.optionsData.find((/**
             * @param {?} item
             * @return {?}
             */
            function (item) { return '' + item.value === '' + value; }));
            return opt ? opt.text : '';
        }
        else if (value === null || value === '' || value === void 0) {
            return null;
        }
    };
    /**
     * @param {?} term
     * @param {?} item
     * @return {?}
     */
    FlexyControlSelect2Component.prototype.customSearchFn = /**
     * @param {?} term
     * @param {?} item
     * @return {?}
     */
    function (term, item) {
        return ((item.text && item.text.toLowerCase().includes(term.toLowerCase())) ||
            (item.value && item.value.toLowerCase().includes(term.toLowerCase())));
    };
    FlexyControlSelect2Component.decorators = [
        { type: Component, args: [{
                    template: "\n    <ng-select\n      *ngIf=\"!readonly && optionsData\"\n      #select\n      [ngClass]=\"{ 'ng-select-multiple': multiple }\"\n      [formControl]=\"control\"\n      [items]=\"optionsData\"\n      [placeholder]=\"placeholder\"\n      bindLabel=\"text\"\n      bindValue=\"value\"\n      [multiple]=\"multiple\"\n      [addTag]=\"addItem\"\n      [hideSelected]=\"hideSelected\"\n      [searchFn]=\"enableSearchByValue ? customSearchFn : null\"\n      [virtualScroll]=\"virtualScroll\"\n      [loading]=\"loading\"\n      [loadingText]=\"loadingText\"\n      (change)=\"onChange($event)\"\n    >\n      <ng-template ng-label-tmp let-item=\"item\" let-clear=\"clear\">\n        <span *ngIf=\"item['prefixHtml']\" [innerHTML]=\"item['prefixHtml']\"></span> <span title=\"{{ item.text }}\">{{ item.text }}</span>\n        <span class=\"ng-value-icon right t2e-unselect-value\" (click)=\"clear(item)\">\u00D7</span>\n      </ng-template>\n      <ng-template ng-option-tmp let-item=\"item\" let-index=\"index\">\n        <span *ngIf=\"item['prefixHtml']\" [innerHTML]=\"item['prefixHtml']\"></span> <span title=\"{{ item.text }}\">{{ item.text }}</span>\n      </ng-template>\n    </ng-select>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"getReadonlyInfo(control?.value)\" [default]=\"default\"> </flexy-control-readonly>\n  ",
                    selector: 'flexy-control-select2',
                    changeDetection: ChangeDetectionStrategy.OnPush
                }] }
    ];
    FlexyControlSelect2Component.propDecorators = {
        control: [{ type: Input }],
        options: [{ type: Input }],
        default: [{ type: Input }],
        readonly: [{ type: Input }],
        multiple: [{ type: Input }],
        addItem: [{ type: Input }],
        placeholder: [{ type: Input }],
        hideSelected: [{ type: Input }],
        enableSearchByValue: [{ type: Input }],
        loading: [{ type: Input }],
        loadingText: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlSelect2Component;
}());
if (false) {
    /** @type {?} */
    FlexyControlSelect2Component.prototype.control;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.options;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.default;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.readonly;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.multiple;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.addItem;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.placeholder;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.hideSelected;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.enableSearchByValue;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.loading;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.loadingText;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.focused;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.clicked;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.changed;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.virtualScroll;
    /** @type {?} */
    FlexyControlSelect2Component.prototype.optionsData;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlTagsComponent = /** @class */ (function () {
    function FlexyControlTagsComponent() {
        this.focused = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlTagsComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.readonlyValue = this.control.value ? this.control.value.join(', ') : this.default ? this.default : null;
    };
    FlexyControlTagsComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <ng-select\n      *ngIf=\"!readonly\"\n      [formControl]=\"control\"\n      [items]=\"options\"\n      [placeholder]=\"placeholder\"\n      [addTag]=\"true\"\n      [multiple]=\"true\"\n      (change)=\"changed.emit($event)\"\n      (focus)=\"focused.emit($event)\"\n    >\n    </ng-select>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"readonlyValue\" [default]=\"default\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-tags'
                }] }
    ];
    FlexyControlTagsComponent.propDecorators = {
        control: [{ type: Input }],
        options: [{ type: Input }],
        default: [{ type: Input }],
        readonly: [{ type: Input }],
        placeholder: [{ type: Input }],
        focused: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlTagsComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlTagsComponent.prototype.control;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.options;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.default;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.placeholder;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.focused;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.changed;
    /** @type {?} */
    FlexyControlTagsComponent.prototype.readonlyValue;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlTextComponent = /** @class */ (function () {
    function FlexyControlTextComponent() {
        this.autocomplete = '';
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
        this.entered = new EventEmitter();
    }
    FlexyControlTextComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <input\n      *ngIf=\"!readonly\"\n      class=\"form-control\"\n      type=\"text\"\n      [formControl]=\"control\"\n      [attr.minLength]=\"minLength\"\n      [attr.maxLength]=\"maxLength\"\n      [attr.placeholder]=\"default\"\n      [attr.autocomplete]=\"autocomplete\"\n      (focus)=\"focused.emit($event)\"\n      (click)=\"clicked.emit($event)\"\n      (change)=\"changed.emit($event)\"\n      (keyup.enter)=\"$event.preventDefault(); entered.emit($event)\"\n    />\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"control.value\" [default]=\"default\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-text'
                }] }
    ];
    FlexyControlTextComponent.propDecorators = {
        control: [{ type: Input }],
        default: [{ type: Input }],
        minLength: [{ type: Input }],
        maxLength: [{ type: Input }],
        readonly: [{ type: Input }],
        autocomplete: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }],
        entered: [{ type: Output }]
    };
    return FlexyControlTextComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlTextComponent.prototype.control;
    /** @type {?} */
    FlexyControlTextComponent.prototype.default;
    /** @type {?} */
    FlexyControlTextComponent.prototype.minLength;
    /** @type {?} */
    FlexyControlTextComponent.prototype.maxLength;
    /** @type {?} */
    FlexyControlTextComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlTextComponent.prototype.autocomplete;
    /** @type {?} */
    FlexyControlTextComponent.prototype.focused;
    /** @type {?} */
    FlexyControlTextComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlTextComponent.prototype.changed;
    /** @type {?} */
    FlexyControlTextComponent.prototype.entered;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlTreeSelectComponent = /** @class */ (function () {
    function FlexyControlTreeSelectComponent() {
        // TODO
        this.focused = new EventEmitter();
        this.clicked = new EventEmitter();
        this.changed = new EventEmitter();
    }
    /**
     * @return {?}
     */
    FlexyControlTreeSelectComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () { };
    /**
     * @param {?} event
     * @return {?}
     */
    FlexyControlTreeSelectComponent.prototype.selectItem = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.control.setValue(event && event.id ? event.id : null);
    };
    FlexyControlTreeSelectComponent.decorators = [
        { type: Component, args: [{
                    template: "\n    <div *ngIf=\"!readonly\">\n      <flexy-tree\n        [tree]=\"dataTree\"\n        [selectedId]=\"control.value\"\n        [readonlyId]=\"readonlyId\"\n        [templateRef]=\"treeRef\"\n        (select)=\"selectItem($event)\"\n      >\n      </flexy-tree>\n      <ng-template #treeRef let-item>{{ item[labelKey] }}</ng-template>\n    </div>\n    <flexy-control-readonly *ngIf=\"readonly\" [value]=\"control.value\" [default]=\"default\"></flexy-control-readonly>\n  ",
                    selector: 'flexy-control-tree-select'
                }] }
    ];
    FlexyControlTreeSelectComponent.propDecorators = {
        control: [{ type: Input }],
        dataTree: [{ type: Input }],
        default: [{ type: Input }],
        readonlyId: [{ type: Input }],
        labelKey: [{ type: Input }],
        readonly: [{ type: Input }],
        focused: [{ type: Output }],
        clicked: [{ type: Output }],
        changed: [{ type: Output }]
    };
    return FlexyControlTreeSelectComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.control;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.dataTree;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.default;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.readonlyId;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.labelKey;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.focused;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.clicked;
    /** @type {?} */
    FlexyControlTreeSelectComponent.prototype.changed;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormsTextareaAutosizeDirective = /** @class */ (function () {
    function FlexyFormsTextareaAutosizeDirective(element) {
        this.element = element;
    }
    /**
     * @param {?} textArea
     * @return {?}
     */
    FlexyFormsTextareaAutosizeDirective.prototype.onInput = /**
     * @param {?} textArea
     * @return {?}
     */
    function (textArea) {
        this.adjust();
    };
    /**
     * @return {?}
     */
    FlexyFormsTextareaAutosizeDirective.prototype.ngAfterContentChecked = /**
     * @return {?}
     */
    function () {
        this.adjust();
    };
    /**
     * @return {?}
     */
    FlexyFormsTextareaAutosizeDirective.prototype.adjust = /**
     * @return {?}
     */
    function () {
        this.element.nativeElement.style.overflow = 'hidden';
        this.element.nativeElement.style.height = 'auto';
        this.element.nativeElement.style.height = this.element.nativeElement.scrollHeight + 'px';
    };
    FlexyFormsTextareaAutosizeDirective.decorators = [
        { type: Directive, args: [{
                    selector: 'textarea[flexyAutosize]'
                },] }
    ];
    /** @nocollapse */
    FlexyFormsTextareaAutosizeDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    FlexyFormsTextareaAutosizeDirective.propDecorators = {
        onInput: [{ type: HostListener, args: ['input', ['$event.target'],] }]
    };
    return FlexyFormsTextareaAutosizeDirective;
}());
if (false) {
    /** @type {?} */
    FlexyFormsTextareaAutosizeDirective.prototype.element;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyFormJsonFileComponent = /** @class */ (function () {
    function FlexyFormJsonFileComponent() {
    }
    FlexyFormJsonFileComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-form-json-file',
                    template: "\n    <flexy-form-field [control]=\"layoutSchema.formControl\" [label]=\"label\" [description]=\"description\">\n      <label role=\"button\">\n        <flexy-control-json-file [control]=\"layoutSchema.formControl\" [readonly]=\"readonly\" [name]=\"label\"></flexy-control-json-file>\n      </label>\n    </flexy-form-field>\n  "
                }] }
    ];
    FlexyFormJsonFileComponent.propDecorators = {
        layoutSchema: [{ type: Input }],
        label: [{ type: Input }],
        description: [{ type: Input }],
        readonly: [{ type: Input }]
    };
    return FlexyFormJsonFileComponent;
}());
if (false) {
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.layoutSchema;
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.label;
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.description;
    /** @type {?} */
    FlexyFormJsonFileComponent.prototype.readonly;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var FlexyControlJsonFileComponent = /** @class */ (function () {
    function FlexyControlJsonFileComponent() {
    }
    /**
     * @return {?}
     */
    FlexyControlJsonFileComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        try {
            this.json = this.control.value ? JSON.parse(this.control.value) : null;
        }
        catch (err) {
            console.error(err);
        }
    };
    /**
     * @param {?} json
     * @return {?}
     */
    FlexyControlJsonFileComponent.prototype.setJson = /**
     * @param {?} json
     * @return {?}
     */
    function (json) {
        this.control.setValue(JSON.stringify(json));
        this.control.markAsDirty();
        this.json = json;
    };
    FlexyControlJsonFileComponent.decorators = [
        { type: Component, args: [{
                    selector: 'flexy-control-json-file',
                    template: "\n    <ng-container *ngIf=\"!readonly\">\n      <flexy-json-import-button class=\"btn btn-primary\" (uploaded)=\"setJson($event)\">Upload</flexy-json-import-button>\n    </ng-container>\n    <ng-container *ngIf=\"json\">\n      <button type=\"button\" flexyJsonExport [name]=\"name\" [json]=\"json\" class=\"btn btn-link\">Download file</button>\n    </ng-container>\n  "
                }] }
    ];
    /** @nocollapse */
    FlexyControlJsonFileComponent.ctorParameters = function () { return []; };
    FlexyControlJsonFileComponent.propDecorators = {
        control: [{ type: Input }],
        readonly: [{ type: Input }],
        name: [{ type: Input }]
    };
    return FlexyControlJsonFileComponent;
}());
if (false) {
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.control;
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.readonly;
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.name;
    /** @type {?} */
    FlexyControlJsonFileComponent.prototype.json;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var FLEXY_FORM_CONTROLS_JSON_MAPPER = {
    number: FlexyFormNumberComponent,
    text: FlexyFormTextComponent,
    select: FlexyFormSelectComponent,
    textarea: FlexyFormTextareaComponent,
    checkbox: FlexyFormCheckboxComponent,
    array: FlexyFormArrayComponent,
    group: FlexyFormGroupComponent,
    fieldset: FlexyFormFieldsetComponent,
    select2: FlexyFormSelect2Component,
    datepicker: FlexyFormDatepickerComponent,
    daterangepicker: FlexyFormDaterangepickerComponent,
    radiolist: FlexyFormRadioListComponent,
    colorpicker: FlexyFormColorpickerComponent,
    tags: FlexyFormTagsComponent,
    checkboxlist: FlexyFormCheckboxListComponent,
    treeselect: FlexyFormTreeSelectComponent,
    label: FlexyFormLabelComponent,
    tabs: FlexyFormTabsComponent,
    percent: FlexyFormPercentComponent,
    password: FlexyFormPasswordComponent,
    chips: FlexyFormChipsComponent,
    draggableselect: FlexyFormDraggableSelectComponent,
    jsonfile: FlexyFormJsonFileComponent
};
/** @type {?} */
var ENTRY_LAYOUT_COMPONENTS = [
    FlexyFormNumberComponent,
    FlexyFormSelectComponent,
    FlexyFormTextComponent,
    FlexyFormTextareaComponent,
    FlexyFormCheckboxComponent,
    FlexyFormArrayComponent,
    FlexyFormGroupComponent,
    FlexyFormFieldsetComponent,
    FlexyFormSelect2Component,
    FlexyFormDatepickerComponent,
    FlexyFormDaterangepickerComponent,
    FlexyFormRadioListComponent,
    FlexyFormColorpickerComponent,
    FlexyFormTagsComponent,
    FlexyFormCheckboxListComponent,
    FlexyFormTreeSelectComponent,
    FlexyFormLabelComponent,
    FlexyFormTabsComponent,
    FlexyFormPercentComponent,
    FlexyFormPasswordComponent,
    FlexyFormChipsComponent,
    FlexyFormDraggableSelectComponent,
    FlexyFormJsonFileComponent
];
/** @type {?} */
var FORM_CONTROLS = [
    FlexyControlCheckboxComponent,
    FlexyControlNumberComponent,
    FlexyControlSelectComponent,
    FlexyControlReadonlyComponent,
    FlexyControlTextComponent,
    FlexyControlSelect2Component,
    FlexyControlDatepickerComponent,
    FlexyControlDaterangepickerComponent,
    FlexyControlRadioListComponent,
    FlexyControlColorpickerComponent,
    FlexyControlTagsComponent,
    FlexyControlCheckboxListComponent,
    FlexyControlTreeSelectComponent,
    FlexyControlPercentComponent,
    FlexyControlPasswordComponent,
    FlexyControlChipsComponent,
    FlexyControlDraggableSelectComponent,
    FlexyControlJsonFileComponent
];
/** @type {?} */
var PUBLIC_COMPONENTS = [FlexyFormsTextareaAutosizeDirective, FlexyFieldControlInfoComponent, FlexyFieldComponent, FlexyTreeComponent];
/** @type {?} */
var COMPONENTS = [FlexyTreeNodeComponent];
var FlexyFormsBootstrapModule = /** @class */ (function () {
    function FlexyFormsBootstrapModule() {
    }
    /**
     * @return {?}
     */
    FlexyFormsBootstrapModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: FlexyFormsBootstrapModule,
            providers: [FlexyTreeService]
        };
    };
    FlexyFormsBootstrapModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule,
                        FormsModule,
                        ReactiveFormsModule,
                        TooltipModule,
                        TabsModule,
                        TranslateModule,
                        FlexyLayoutModule,
                        FlexyFormsModule,
                        NgSelectModule,
                        TagInputModule,
                        BsDatepickerModule,
                        FlexyJsonImpExpModule,
                        DragDropModule
                    ],
                    declarations: __spread(COMPONENTS, PUBLIC_COMPONENTS, ENTRY_LAYOUT_COMPONENTS, FORM_CONTROLS),
                    entryComponents: ENTRY_LAYOUT_COMPONENTS,
                    exports: __spread(PUBLIC_COMPONENTS, ENTRY_LAYOUT_COMPONENTS, FORM_CONTROLS)
                },] }
    ];
    return FlexyFormsBootstrapModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FLEXY_FORM_CONTROLS_JSON_MAPPER, FlexyControlCheckboxComponent, FlexyControlCheckboxListComponent, FlexyControlChipsComponent, FlexyControlColorpickerComponent, FlexyControlDatepickerComponent, FlexyControlDaterangepickerComponent, FlexyControlDraggableSelectComponent, FlexyControlNumberComponent, FlexyControlPasswordComponent, FlexyControlPercentComponent, FlexyControlRadioListComponent, FlexyControlReadonlyComponent, FlexyControlSelect2Component, FlexyControlSelectComponent, FlexyControlTagsComponent, FlexyControlTextComponent, FlexyControlTreeSelectComponent, FlexyFieldComponent, FlexyFieldControlInfoComponent, FlexyFormArrayComponent, FlexyFormCheckboxComponent, FlexyFormCheckboxListComponent, FlexyFormChipsComponent, FlexyFormColorpickerComponent, FlexyFormDatepickerComponent, FlexyFormDaterangepickerComponent, FlexyFormDraggableSelectComponent, FlexyFormFieldsetComponent, FlexyFormGroupComponent, FlexyFormLabelComponent, FlexyFormNumberComponent, FlexyFormPasswordComponent, FlexyFormPercentComponent, FlexyFormRadioListComponent, FlexyFormSelect2Component, FlexyFormSelectComponent, FlexyFormTabsComponent, FlexyFormTagsComponent, FlexyFormTextComponent, FlexyFormTextareaComponent, FlexyFormTreeSelectComponent, FlexyFormsBootstrapModule, FlexyFormsTextareaAutosizeDirective, FlexyTreeComponent, FlexyTreeNodeComponent, FlexyTreeService, FlexyFormJsonFileComponent as ɵa, FlexyControlJsonFileComponent as ɵb };
//# sourceMappingURL=ng-flexy-form-bootstrap.js.map
